<?php
/**
 * File delle funzioni di sistema
 *
 * Questo file contiene le procedure di sistema necessarie al funzionamento di FlatNux.
 *
 * @package Flatnux_functions
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @copyright Copyright (c) 2003-2005
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License
 */
require_once ("include/functions_php.php");
require_once ("include/functions_xmldb.php");
require_once ("include/functions_javascripts.php");
require_once ("include/functions_editors.php");
require_once ("include/shared.php");
//---xmldb--->
require_once ("include/xmldb.php");
require_once ("include/xmldb_forms.php");
require_once ("include/xmldb_mysql.php");
require_once ("include/xmldb_serialize.php");
require_once ("include/xmldb_csv.php");
require_once ("include/xmldb_editor.php");
require_once ("include/xmldb_query.php");
//---xmldb---<
require_once ("include/php_filters/sanitize.php");
require_once ("include/admin.php");
/**
 * Visualizza una sezione
 *
 * Visualizza il contenuto della sezione passata come parametro.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $section Nome assoluto della sezione da visualizzare
 * @param bool $showsgraphics visualizza footer e cornice sessione
 *
 */
function view_section($section, $showsgraphics = true)
{
	global $_FN;
	$getfile = getparam("file", PAR_GET, SAN_HTML);
	if ( strstr($getfile, "..") )
	{
		$getfile = "";
	}
	if ( strstr($section, "..") )
	{
		$section = "";
	}
	$lang = $_FN['lang'];
	if ( $section == "" || !is_dir("sections/$section") )
	{
		echo (_SECTNOTEXIST);
		jsredirect("index.php");
		return;
	}
	$file = $getfile;
	//------ CONTROLLO PERMESSI -->
	if ( !user_can_view_section($_FN['vmod'], $_FN['user']) )
	{
		$sectlevel = getsectlevel($section);
		OpenTable();
		if ( $_FN['user'] == "" && $sectlevel == 0 ) // se il livello e' 0 basta essere registrati per visualizzare la sezione
		{
			echo _FERRACC . "<br /><br />";
			fn_loginform();
			echo "<br /><br /><a href=\"javascript:history.back()\">&lt;&lt;" . _INDIETRO . "</a>";
		}
		else
			echo ("<div align='center'><b>" . _NOLEVELSECT . "</b></div>");
		CloseTable();
		return;
	}
	//------ CONTROLLO PERMESSI --<
	// procedura per barra di navigazione -->
	if ( $file != "" )
		$sect = $section . "/$file";
	else
		$sect = $section;
	$albero = explode("/", $sect);
	for ( $i = 0; $i < sizeof($albero); $i++ )
	{
		$mypath = "";
		for ( $j = 0; $j <= $i; $j++ )
		{
			if ( $i > 0 and ($j != $i) )
			{
				$mypath .= $albero[$j] . "/";
			}
			else
			{
				$mypath .= $albero[$j];
			}
		}
		$tmp = str_replace("none_", "", $albero[$i]);
		$tmp = preg_replace("/^[0-9][0-9]_/si", "", $tmp);
		$tmp = str_replace("_", "&nbsp;", $tmp);
		$tmp = getLang("sections/$mypath/lang.xml", $tmp);
		$titoli[$i] = $tmp;
	}
	$tit = FN_CreateNavbar($albero, $titoli);
	// <-- fine procedura per barra di navigazione
	if ( $showsgraphics )
		FN_OpenSection($tit);
	if ( $file == "" && file_exists('sections/' . $section . '/section.php') )
	{
		include ('sections/' . $section . '/section.php');
	}
	else
	if ( $file == "" )
	{
		if ( file_exists("sections/" . $section . "/section.$lang.html") )
		{ //--lingua corrente
			echo link_local_to_section('sections/' . $section . "/section.$lang.html");
		}
		else
		{
			$filefound = false;
			if ( !($_FN['lang_auto_switch'] === 0) )
			{
				// ----se la pagina non esiste nella lingua di default provo con le altre lingue ----
				foreach ( $_FN['listlanguages'] as $l )
				{
					if ( file_exists("sections/" . $section . "/section.$l.html") )
					{
						echo link_local_to_section('sections/' . $section . "/section.$l.html");
						$filefound = true;
						break;
					}
				}
			}
			if ( !$filefound && $_FN['enable_online_administration'] )
			{
				show_create_file_section($section);
			}
		}
	}
	else
	{
		if ( file_exists('sections/' . $section . '/' . $file) )
		{
			if ( get_file_extension($file) == 'php' )
				include ('sections/' . $section . '/' . $file);
			else
				echo link_local_to_section('sections/' . $section . '/' . $file);
		}
		elseif ( $_FN['enable_online_administration'] )
		{
			show_create_file_custom($section, $file);
		}
	}
	if ( $showsgraphics )
	{
		FN_CloseSection($tit);
	}
}
/**
 * Effettua il parsing di un feed RSS esterno
 *
 * Legge il file di un feed RSS esterno al sito e ne genera una stampa
 * leggibile e ordinata con le notizie segnalate, completa di link
 * per raggiungerle sul sito di origine.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $url Indirizzo del feed RSS
 * @return null|string Nulla se non trova il file, oppure il contenuto del feed RSS
 */
function parse_RSS($url)
{
	$url = getparam($url, PAR_NULL, SAN_FLAT);
	if ( $url != "" )
	{
		if ( !preg_match("/http:\\/\\//i", $url) )
		{
			$url = "http://$url";
		}
		$rdf = parse_url($url);
		$errno = $errstr = null;
		$fp = fsockopen($rdf['host'], 80, $errno, $errstr, 15);
		if ( !$fp )
		{
			print(_NORSS);
			return "";
		}
		if ( $fp )
		{
			$remotecontents = "";
			@ fputs($fp, "GET " . $rdf['path'] . "?" . $rdf['query'] . " HTTP/1.0\r\n");
			@ fputs($fp, "HOST: " . $rdf['host'] . "\r\n\r\n");
			while ( !feof($fp) )
			{
				$pagetext = fgets($fp, 228);
				$remotecontents .= chop($pagetext);
			}
			fputs($fp, "Connection: close\r\n\r\n");
			fclose($fp);
			//-----------------ricava e converte l' encoding----------->
			$encoding = 'UTF-8';
			$arr_got = array();
			if ( preg_match('/encoding="([^"]+)"/iUs', $remotecontents, $arr_got) )
				$encoding = strtoupper($arr_got[1]);
			$string = $remotecontents;
			$remotecontents = htmlentities($remotecontents, ENT_QUOTES, $encoding);
			$remotecontents = html_entity_decode($remotecontents, ENT_QUOTES, _CHARSET);
			if ( $remotecontents != "" )
				$string = $remotecontents;
			echo "<br />";
			//-----------------ricava e converte l' encoding-----------<
			$string = xmldb_removexmlcomments($string);
			$description = "";
			$tmp = explode("</title>", $string);
			$title = preg_replace("/.*<title>/si", "", $tmp[0]);
			$tmp = explode("</description>", preg_replace("/<item>.*<\/item>/si", "", $string));
			if ( isset($tmp[1]) )
				$description = preg_replace("/.*<description>/si", "", $tmp[0]);
			$items = explode("</item>", $string);
			$content = "" . "<strong>$title</strong><br /><em>$description</em><hr /><div>";
			for ( $i = 0; $i < sizeof($items) - 1; $i++ )
			{
				$link = preg_replace("/.*<link>/si", "", $items[$i]);
				$link = preg_replace("/<\\/link>.*/si", "", $link);
				$title2 = preg_replace("/.*<title>/si", "", $items[$i]);
				$title2 = preg_replace("/<\\/title>.*/si", "", $title2);
				// some feeds are exposed with CDATA format
				$title2 = str_replace("<![CDATA[", "", $title2);
				$title2 = str_replace("]]>", "", trim($title2));
				if ( strcmp($link, $title2) and $items[$i] != "" )
				{
					$content .= "<b><big>&middot;</big></b>&nbsp;<a style=\"font-size: 12px;\" href=\"$link\" target=\"new\">" . $title2 . "</a><br />\n";
				}
			}
			$content .= "</div>";
		}
	}
	return $content;
}
/**
 * Restituisce il livello di una sezione
 *
 * Restituisce il livello della sezione passata come parametro, leggendolo dal
 * file <i>/section/nomesezione/level.php</i>:
 * - -1 se non esiste il file level.php, la sezione sara' visibile da tutti;
 * - 0 per una sezione visibile solo da utenti registrati;
 * - da 1 a 9 per una sezione visibile solo da utenti di livello intermedio;
 * - 10 per una sezione visibile solo da utenti amministratore.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $section Nome della sezione
 * @return string -1 se la sezione non ha livello, oppure un numero di livello da 0 a 10
 */
function getsectlevel($section)
{
	if ( !@file_exists("sections/$section/level.php") )
		return ( -1);
	$perms = getsectperm("sections/$section");
	return $perms['level'];
}
/**
 * Restituisce i permessi di una sezione
 *
 * Restituisce i permessi della sezione passata come parametro, leggendolo dal
 * file <i>/section/nomesezione/level.php</i>:
 * - -1 se non esiste il file level.php, la sezione sara' visibile da tutti;
 * - 0 per una sezione visibile solo da utenti registrati;
 * - da 1 a 9 per una sezione visibile solo da utenti di livello intermedio;
 * - 10 per una sezione visibile solo da utenti amministratore.
 *
 * @author Alessandro Vernassa <speleoalex@fastwebnet.it>
 *
 * @param string $section cartella
 * @return array
 */
function getsectperm($section)
{
	static $cacheperm = array();
	if ( isset($cacheperm[$section]) )
	{
		return $cacheperm[$section];
	}
	if ( !@file_exists("$section/level.php") )
	{
		$perm['level'] = -1;
		$perm['user'] = "";
		$perm['group'] = "";
		$perm['user_view'] = "";
		$perm['group_view'] = "";
		$cacheperm[$section] = $perm;
		return $perm;
	}
	$fp = file("$section/level.php");
	$level = str_replace("\r", "", str_replace("\n", "", $fp[0]));
	//retrocompatibilita'
	if ( $level != "" && strstr("1023456789", $level) )
	{
		$perm['level'] = intval($level);
		$perm['user'] = "";
		$perm['group'] = "";
		$perm['user_view'] = "";
		$perm['group_view'] = "";
	}
	else
	{
		$level = str_replace("\r", "", str_replace("\n", "", $fp[1]));
		$user = str_replace("\r", "", str_replace("\n", "", isset($fp[2]) ? $fp[2] : ""));
		$group = str_replace("\r", "", str_replace("\n", "", isset($fp[3]) ? $fp[3] : ""));
		$user_view = str_replace("\r", "", str_replace("\n", "", isset($fp[4]) ? $fp[4] : ""));
		$group_view = str_replace("\r", "", str_replace("\n", "", isset($fp[5]) ? $fp[5] : ""));
		$perm['level'] = $level;
		$perm['user'] = $user;
		$perm['group'] = $group;
		$perm['user_view'] = $user_view;
		$perm['group_view'] = $group_view;
	}
	$cacheperm[$section] = $perm;
	return $perm;
}
/**
 * Restituisce il livello di una cartella
 *
 * Restituisce il livello della cartella passata come parametro, leggendolo dal
 * file <i>/nomecartella/level.php</i>:
 * - -1 se non esiste il file level.php, la cartella sara' visibile da tutti;
 * - 0 per una cartella visibile solo da utenti registrati;
 * - da 1 a 9 per una cartella visibile solo da utenti di livello intermedio;
 * - 10 per una cartella visibile solo da utenti amministratore.
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param string $dir Nome della cartella
 * @return string -1 se la cartella non ha livello, oppure un numero di livello da 0 a 10
 */
function getdirlevel($dir)
{
	if ( !file_exists("$dir/level.php") )
		return ( -1);
	$fp = file("$dir/level.php");
	$level = str_replace("\r", "", str_replace("\n", "", $fp[0]));
	//retrocompatibilita'
	if ( !strstr($level, "1023456789") )
		return $level;
	else
	{
		$level = str_replace("\r", "", str_replace("\n", "", $fp[1]));
		return $level;
	}
}
/**
 * Funzione alternativa a html_entity_decode()
 *
 * Rimpiazza la funzione nativa di PHP html_entity_decode() per gli utenti
 * che utilizzano una versione di PHP piu' vecchia della 4.3.0.
 *
 * @author Documentazione ufficiale PHP - {@link http://it.php.net/manual/en/function.html-entity-decode.php}
 *
 * @param string $string
 * @return string Una stringa che preserva i caratteri speciali HTML
 */
function unhtmlentities($string)
{
	$trans_tbl = get_html_translation_table(HTML_ENTITIES);
	$trans_tbl = array_flip($trans_tbl);
	return strtr($string, $trans_tbl);
}
/**
 * Genera un log di attivita'
 *
 * @param string $zone Nome della sezione cui si riferisce il log
 * @param string $txt Testo del log da salvare
 */
function FN_log($txt)
{
	global $_FN;
	$flog = "{$_FN['datadir']}/log"; //splx zone da forum non c'e' piu'
	if ( !is_dir($flog) )
		mkdir($flog, 0777);
	$datelog = date("Y-m");
	if ( !file_exists("$flog/$datelog-log.php") )
	{
		$fp = fopen("$flog/$datelog-log.php", "a");
		fwrite($fp, "<?php exit(1);?>\n");
		fclose($fp);
	}
	if ( is_writable("$flog/$datelog-log.php") )
	{
		$fp = fopen("$flog/$datelog-log.php", "a");
		$ip = htmlspecialchars(getparam("REMOTE_ADDR", PAR_SERVER, SAN_FLAT));
		$user = isset($_FN['user']) ? $_FN['user'] : "";
		$txt = str_replace('"', '""', $txt);
		$ip = str_replace('"', '""', $ip);
		$user = str_replace('"', '""', $user);
		$string = xmldb_now() . ";\"$ip\";\"{$_FN['self']}\";\"{$_FN['vmod']}\";\"$user\";\"$txt\";\n";
		fwrite($fp, $string);
		fclose($fp);
	}
	else
	{
		if ( is_admin ( ) )
		{
			echo "<br />";
			echo "$flog/$datelog-log.php:" . fn_i18n("_READONLY") . "<br />";
		}
	}
	//notifica via email all' amministratore
	if ( $_FN['enable_log_email'] == 1 )
	{
		$txtmail = "Log from: {$_FN['sitename']}";
		$txtmail .= "\r\n\r\nSite url:{$_FN['siteurl']}";
		$txtmail .= "\r\n\r\nLog: $string";
		@mail($_FN['log_email_address'], "[fnlog] {$_FN['sitename']}", $txtmail, "From: {$_FN['site_email_address']}\r\n" . "Reply-To: {$_FN['site_email_address']}\r\n" . "X-Mailer: PHP/" . phpversion());
	}
}
/**
 * Mostra le informazioni sul copyright di un modulo
 *
 * Crea un link da cui e' possibile visualizzare una finestra popup
 * con informazioni dettagliate sul copyright del modulo corrente.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @since 2.5.4
 *
 * @param string $modulo Nome del modulo installato
 * @param string $versione Versione del modulo installato
 * @param string $autore Nome dell'autore del modulo
 * @param string $email E-mail dell'autore del modulo
 * @param string $homepage Homepage dell'autore del modulo
 * @param string $licenza Licenza di rilascio del modulo
 */
function module_copyright($modulo, $versione, $autore, $email, $homepage, $licenza)
{
	echo "<script language='JavaScript' type='text/JavaScript'>
var dom = (document.getElementById) ? true : false;
var ns5 = ((navigator.userAgent.indexOf('Gecko')>-1) && dom) ? true: false;
var ie5 = ((navigator.userAgent.indexOf('MSIE')>-1) && dom) ? true : false;
var ns4 = (document.layers && !dom) ? true : false;
var ie4 = (document.all && !dom) ? true : false;
var nodyn = (!ns5 && !ns4 && !ie4 && !ie5) ? true : false;

window.onload=function(){
	tooltip = (ns4)? document.fncopyright.document: (ie4)? document.all['fncopyright']: (ie5||ns5)? document.getElementById('fncopyright'): null;
	tipcss = (ns4)? document.fncopyright: tooltip.style;

	if(dom && ns5) {
		tipcss.position = 'fixed';
	}
	else {
		tipcss.position = 'absolute';
	}
}

function copyshow(){

	tooltip = (ns4)? document.fncopyright.document: (ie4)? document.all['fncopyright']: (ie5||ns5)? document.getElementById('fncopyright'): null;
	tipcss = (ns4)? document.fncopyright: tooltip.style;

	if(tipcss.visibility=='visible')
		tipcss.visibility='hidden';
	else
		tipcss.visibility='visible';
}

</script>";
	echo "<br /><div align='right'><a href='javascript:;' onclick='copyshow();'>Copyright &copy; <b>$modulo</b></a></div><br />";
	echo "<div id='fncopyright' style='top: 15px; left: 15px; display: block; visibility: hidden; background-color: #FFFBA6; border: 1px solid; padding: 5px;'>
								<p align='center'><b>Copyright informations</b></p>Module developed for the <b><a href='http://flatnuke.sf.net'>flatnuke</a></b> CMS<br /><br /><b>Module name</b>: $modulo<br /><b>Version</b>: $versione<br /><b>License</b>: $licenza<br /><b>Author</b>: $autore<br /><b>E-mail</b>: <a href='mailto:$email'>$email</a><br /><b>Home page</b>: <a href='$homepage' target='_blank' alt='$homepage' title='$homepage'>$homepage</a> <br /><p align='center'><b><a href='javascript:;' onclick='copyshow();'>Close</a></b></p></div>";
}
/**
 * Crea una nuova pagina
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $section percorso della sessione
 *
 * */
function new_file($filename, $section)
{
	global $_FN;
	if ( can_modify_file($_FN['user'], "sections/$section/$filename") )
	{
		if ( file_exists("sections/" . $section) && is_dir("sections/" . $section) )
		{
			touch("sections/" . $section . "/" . $filename);
			//verifico l' esistenza del file creato
			if ( file_exists("sections/" . $section . "/" . $filename) )
			{
				jsalert(_FILECREATED . ":\nsections/" . $section . "/" . $filename);
				jsredirect("index.php?opindex=modcont&file=sections/$section/$filename&from=index.php?mod=$section");
			}
			else
			{
				jsalert(_FILENOTCREATED . ":\nsections/" . $section . "/" . $filename);
			}
		}
	}
}
/**
 * converte in un file i links da locali riferiti alla sezione
 * e lascia invariati gli altri
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param $filename nome del file da convertire
 * @return stringa convertita
 *
 * */
function link_local_to_section($filename)
{
	global $_FN;
	$mod = getparam("mod", PAR_GET, SAN_FLAT);
	if ( !file_exists($filename) )
	{
		return "";
	}
	//return file_get_contents($filename);
	$sdir = dirname($filename);
	$url = $_FN['siteurl'];
	$sdir = $url . $sdir;
	//die($sdir);
	$getfile = getparam("file", PAR_GET, SAN_HTML);
	if ( strstr($getfile, "..") )
	{
		$getfile = "";
	}
	$ret = "";
	$fp = fopen($filename, "r");
	if ( !$fp )
		return "";
	//$protocols="mms:|skype:|javascript:|/|https:|http:|ftp:|mailto:";
	while ( !feof($fp) )
	{
		/*  da provare:
			$line = preg_replace("/<([^>]+)( background| href| src)=(\"|')(.*)/si", "<\\1\\2=\\3$sdir\\4", $line);
		 */
		$line = fgets($fp, 1024);
		//Proteggo i link esterni dal cambiamento (apici)
		$line = preg_replace("/( background| href| src)='(mms:|skype:|javascript:|\\/|https:\\/\\/|http:\\/\\/|ftp:\\/\\/|mailto:|index.php?)/si", "\\1ex='\\2", $line);
		//Proteggo i link esterni dal cambiamento (doppi apici)
		$line = preg_replace("/( background| href| src)=\"(mms:|skype:|javascript:|\\/|https:\\/\\/|http:\\/\\/|ftp:\\/\\/|mailto:|index.php?)/si", "\\1ex=\"\\2", $line);
		//Trasformo link a sezioni dello stesso file (es.: href='#ciao')
		if ( $getfile != "" ) //se e' un file all' interno di una section
			$line = preg_replace("/ href='(#.+)'/si", " hrefex='index.php?mod=$mod&amp;file=$getfile\\1'", $line);
		else
			$line = preg_replace("/( href| src)='(mms:|skype:|javascript:|https:\/\/|http:\/\/|ftp:\/\/|mailto:|index.php?)/si", "\\1ex='\\2", $line);
		//Trasformo link a sezioni dello stesso file (es.: href="#ciao")
		if ( $getfile != "" ) //se e' un file all' interno di una section
			$line = preg_replace("/ href=\"(#.+)\"/si", " hrefex=\"index.php?mod=$mod&amp;file=$getfile\\1\"", $line);
		else
			$line = preg_replace("/( href| src)=\"(mms:|skype:|javascript:|https:\/\/|http:\/\/|ftp:\/\/|mailto:|index.php?)/si", "\\1ex=\"\\2", $line);
		//Trasformo link del tipo "(mio.php" devo evitare di modificare i limk assoluti
		//se e' section.xx.html
		$line = preg_replace("/ href='(#.+)'/si", " hrefex='index.php?mod=$mod\\1'", $line);
		//se e' section.xx.html
		$line = preg_replace("/ href=\"(#.+)\"/si", " hrefex=\"index.php?mod=$mod\\1\"", $line);
		//Trasformo link ad altre pagine del tipo (es.: href='quella.htm#ciao')
		$line = preg_replace("/ href='([^']+)(\\.htm|html|php|swf|txt+)(['|(#.')])/si", " hrefex='index.php?mod=$mod&amp;file=\\1\\2\\3", $line);
		//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
		$line = preg_replace("/ href=\"([^\"]+)(\\.htm|html|php|swf|txt+)([\"|(#.\")])/si", " hrefex=\"index.php?mod=$mod&amp;file=\\1\\2\\3", $line);
		//Trasformo link ad altre pagine del tipo (es.: href='quella.htm#ciao')
		$line = preg_replace("/ href='([^']+)(\\.jpg|jpeg|bmp|svg|swf|png|gif|txt+)(['|(#.')])/si", " hrefex='$sdir/\\1\\2\\3", $line);
		//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
		$line = preg_replace("/ href=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif|txt+)([\"|(#.\")])/si", " hrefex=\"$sdir/\\1\\2\\3", $line);
		//Trasformo link a file locali (es.: href='mio.tar.gz')
		$line = preg_replace("/ href='([^']+)'/si", " hrefex='$sdir/\\1'", $line);
		//Trasformo link a file locali (es.: href="mio.tar.gz")
		$line = preg_replace("/ href=\"([^\"]+)\"/si", " hrefex=\"$sdir/\\1\"", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src='bella.jpg')
		$line = preg_replace("/ src='([^']+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[']/si", " src='$sdir/\\1\\2'", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ src=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\"]/si", " src=\"$sdir/\\1\\2\"", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src='bella.jpg')
		$line = preg_replace("/ background='([^']+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[']/si", " background='$sdir/\\1\\2'", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ background=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\"]/si", " background=\"$sdir/\\1\\2\"", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ url\\(([^\\)]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\\)]/si", " url($sdir/\\1\\2)", $line);
		//Ripristino le sorgenti che avevo trasformato e protetto
		$line = preg_replace("/( href| src)ex='/si", "\\1='", $line);
		//Ripristino le sorgenti che avevo trasformato e protetto
		$line = preg_replace("/( background| href| src)ex=\"/si", "\\1=\"", $line);
		$ret .= $line;
	}
	fclose($fp);
	return $ret;
}
/**
 * interpreta la pagina nel caso vi sia contenuto del codice php
 *
 * @author Roberto A. Foglietta <me@roberto.foglietta.name>
 * @param string $filename nome del file da convertire
 * @return stringa convertita
 *
 * */
function include_file_to_buffer($filename)
{
	$rows = NULL;
	if ( is_file($filename) )
	{
		if ( preg_match('/\.php$/si', $filename) )
		{
			if ( ob_start ( ) )
			{
				fn_include($filename);
				$string = ob_get_contents();
				$rows = explode("\n", $string);
				ob_end_clean();
			}
		}
		else
		{
			$rows = file($filename);
		}
	}
	return $rows;
}

/**
 * Conto quanti utenti sono registrati
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param int number users
 *
 * */
function count_users()
{
	global $_FN;
	if ( !file_exists("{$_FN['datadir']}/fndatabase/users.php") )
		return 0;
	$table = new XMLTable("fndatabase", "users", $_FN['datadir']);
	return $table->GetNumRecords();
}
/**
 * converte i caratteri speciali in entit xml
 *
 * @param string $string
 * @return string
 */
function xml_special_chars($string)
{
	static $asc2uni = null;
	if ( !is_array($asc2uni) )
	{
		$asc2uni = array();
		for ( $i = 128; $i < 256; $i++ )
		{
			$asc2uni[chr($i)] = "&#x" . dechex($i) . ";";
		}
	}
	$string = str_replace("&", "&amp;", $string);
	$string = str_replace("<", "&lt;", $string);
	$string = str_replace(">", "&gt;", $string);
	$string = str_replace("'", "&apos;", $string);
	$string = str_replace("\"", "&quot;", $string);
	$string = str_replace("\r", "", $string);
	$string = strtr($string, $asc2uni);
	return $string;
	//--conversione in xml------------<
}
/**
 * Setta il titolo di una sezione in una lingua
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $filename file da scrivere
 * @param string $title nuovo titolo sezione
 * @param string $nlang lingua del titolo
 *
 * */
function set_section_title($filename, $title, $nlang)
{
	if ( !is_writable(dirname($filename)) ) //se non e' scrivibile esce
		return false;
	if ( !file_exists($filename) ) //se non esiste esce
		touch($filename);
	if ( !is_writable($filename) )
		return false;
	$str = file_get_contents($filename);
	//$title = htmlentities($title,ENT_QUOTES,_CHARSET);
	$title = htmlentities($title, ENT_QUOTES, _CHARSET);
	$title = html_entity_decode($title, ENT_QUOTES, "UTF-8");
	//$title = htmlentities($title,ENT_QUOTES,"UTF-8");
	//$title = htmlentities($title,ENT_QUOTES,"UTF-8");
	//--conversione in xml------------>
	//$title = xml_special_chars ( $title );
	//--conversione in xml------------<
	if ( strstr($str, "<title>") != false )
	{
		if ( strstr($str, "<$nlang>") != false )
		{
			$start = strpos($str, "<$nlang>");
			$stop = (strpos($str, "</$nlang>")) + 3 + strlen($nlang);
			$oldtitle = substr($str, $start, ($stop - $start));
			$str = str_replace("$oldtitle", "<$nlang>$title</$nlang>", $str);
		}
		else
		{
			$str = str_replace("</title>", "\t<$nlang>$title</$nlang>\n</title>", $str);
		}
	}
	else
	{
		$str = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<title>\n\t<$nlang>$title</$nlang>\n</title>";
	}
	if ( fn_is_external_referer ( ) )
	{
		echo (fn_i18n("_NONPUOI"));
		return;
	}
	$hfile = fopen($filename, "w");
	fwrite($hfile, $str);
	fclose($hfile);
	return true;
}
/**
 * Genera un file html statico con la mappa del sito
 * Utile per indicizzare dai motori di ricerca le
 * pagine php.
 *
 * @param string $indexpath path del file da scrivere
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 */
function write_static_index($indexpath)
{
	global $_FN;
	$lang = $_FN['lang'];
	$sitename = $_FN['sitename'];
	$sitekeywords = $_FN['keywords'];
	$theme = $_FN['theme_default'];
	$sBasePath = $_FN['siteurl'];
	$handle = fopen($indexpath, "w");
	fwrite($handle, "<title>$sitename</title>");
	fwrite($handle, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
	fwrite($handle, "<html lang=\"$lang\">\n");
	fwrite($handle, "<head>\n");
	fwrite($handle, "<title>$sitename</title>\n");
	fwrite($handle, "<meta HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=" . _CHARSET . "\">\n");
	fwrite($handle, "<meta HTTP-EQUIV=\"EXPIRES\" CONTENT=\"0\">\n");
	fwrite($handle, "<meta NAME=\"RESOURCE-TYPE\" CONTENT=\"DOCUMENT\">\n");
	fwrite($handle, "<meta NAME=\"DISTRIBUTION\" CONTENT=\"GLOBAL\">\n");
	fwrite($handle, "<meta NAME=\"AUTHOR\" CONTENT=\"$sitename\">\n");
	fwrite($handle, "<meta NAME=\"COPYRIGHT\" CONTENT=\"Copyright (c) 2005 by $sitename\">\n");
	fwrite($handle, "<meta NAME=\"KEYWORDS\" CONTENT=\"$sitekeywords\">\n");
	fwrite($handle, "<meta NAME=\"DESCRIPTION\" CONTENT=\"$sitename\">\n");
	fwrite($handle, "<meta NAME=\"ROBOTS\" CONTENT=\"INDEX, FOLLOW\">\n");
	fwrite($handle, "<meta NAME=\"REVISIT-AFTER\" CONTENT=\"1 DAYS\">\n");
	fwrite($handle, "<meta NAME=\"RATING\" CONTENT=\"GENERAL\">\n");
	fwrite($handle, "<link rel=\"StyleSheet\" href=\"$sBasePath/themes/$theme/style.css\" type=\"text/css\">\n");
	fwrite($handle, "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"$sitename\" href=\"$sBasePath/{$_FN['datadir']}/$lang/backend.xml\">");
	fwrite($handle, "\n</head>\n<a href='{$sBasePath}index.php' ><h1>$sitename</h1></a><br />\n");
	fclose($handle);
	$str = "";
	foreach ( $_FN['listlanguages'] as $langw )
	{
		//dprint_r($sections);
		$l = "&amp;lang=$langw";
		$l2 = "?lang=$langw";
		if ( $langw == $_FN['lang_default'] )
			$l = $l2 = "";
		$str .= "\n<h2>$langw :</h2>";
		$str .= "<a href=\"index.php$l2\">" . $_FN['sitename'] . "</a>";
		$sections = list_sections_translated("sections", false, true, $langw);
		foreach ( $sections as $section )
		{
			$str .= "\n<br /><img alt=\"{$section['title']}\" src=\"" . fromtheme("images/section.png") . "\" />&nbsp;<a accesskey=\"{$section['accesskey']}\" href=\"{$_FN['siteurl']}/index.php?mod={$section['link']}$l\">{$section['title']}</a>";
		}
	}
	$handle = fopen($indexpath, "a");
	fwrite($handle, $str);
	fclose($handle);
	return;
}
/**
 * Visualizza i crediti di Flatnuke
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * */
function ShowCredits()
{
	global $_FN;
?>
	<a href="http://flatnux.sourceforge.net" onclick="window.open(this.href);return false;" > <img
			align="middle" border="0" src="images/validate/flatnuke_powered.png"
			alt="Powered by FlatNuX!" title="Powered by FlatNuX!" /></a>
	<a href="http://validator.w3.org/check/referer" onclick="window.open(this.href);return false;"> <img
			align="middle" border="0" src="images/validate/valid_html401.png"
			alt="Valid HTML 4.01!" title="Valid HTML 4.01!" /></a>
	<a href="http://jigsaw.w3.org/css-validator/check/referer"
	   onclick="window.open(this.href);return false;" > <img align="middle" border="0"
														  src="images/validate/valid_css.png" alt="Valid CSS!" title="Valid CSS!" /></a>
	<a
		href="<?php echo $_FN['datadir'] ?>/<?php echo $_FN['lang'] ?>/backend.xml"
		onclick="window.open(this.href);return false;" > <img align="middle" border="0"
														  src="images/validate/rss20_powered.png" alt="Get RSS 2.0 Feed"
														  title="Get RSS 2.0 Feed" /></a>
	<br />
		Powered&nbsp;by&nbsp;
	<b><a href="http://flatnux.sourceforge.net">FlatNuX</a></b>
	&nbsp;&copy;&nbsp;2003-2005| Site&nbsp;Admin:&nbsp;
	<a href="mailto:<?php echo $_FN['admin_mail'] ?>"><?php echo $_FN['admin'] ?></a>
	|
	<a href="<?php echo $_FN['datadir'] ?>/index.html">Full&nbsp;Map</a>
<?php
	if ( file_exists("{$_FN['datadir']}/{$_FN['lang']}/backend.xml") )
	{
?>
					|
		<a
			href="<?php echo $_FN['datadir'] ?>/<?php echo $_FN['lang'] ?>/backend.xml">Get&nbsp;RSS&nbsp;News</a><?php
	}
?><br /><?php echo _LEGAL ?>
<?php
}
/**
 * Estrae la stringa nella lingua corrente da un file .xml
 *
 *
 * @author Alessandro Vernassa
 * @param string $filename file xml da cui estrarre la lingua
 * @param string $title titolo originale non tradotto
 * @param string $forcelang lingua da cercare, se null prende quella
 * attualmente impostata
 * @return stringa tradotta
 */
function getLang($filename, $title = null, $forcelang = null, $charsetout = "")
{
	global $_FN;
	$lang = $_FN['lang'];
	if ( $charsetout == "" )
		$charsetout = _CHARSET;
	static $cache = array();
	if ( $forcelang != null )
		$lang_ = $forcelang;
	else
		$lang_ = $lang;
	//evito di cercare piu volte lo stesso titolo
	if ( isset($cache[$filename][$lang_][$charsetout]) )
		return $cache[$filename][$lang_][$charsetout];
	if ( file_exists($filename) )
	{
		$fd = get_file($filename);
		$tmp = get_xml_element($lang_, $fd);
		if ( $tmp != NULL )
		{
			$title = $tmp;
			if ( $charsetout != "UTF-8" )
			{
				$title = htmlentities($tmp, ENT_QUOTES, "UTF-8");
				$title = html_entity_decode($title, ENT_QUOTES, $charsetout);
			}
		}
		$cache[$filename][$lang_][$charsetout] = $title;
	}
	return $title;
}
/**
 * Ricava la lista delle sezioni all' interno di una cartella
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return array che contiene la lista delle sezioni, per scorrerlo utilizzare foreach
 * 
 * @param string $path Directory delle sezioni da scorrere
 * @param string $showhidden mostra le sezioni nascoste.Per compatibilita' con la nuova 
 * gestione temi di fn 2.6.8 $showhidden puo' ricevere in ingresso una 
 * stringa links o titoli che  Specifica
 * 
 * se restituire i nomi delle sezioni ('names') oppure i loro link ('links')
 * 
 */
function list_sections($path = "sections", $showhidden = false, $recursive = false)
{
	global $_FN;
	if ( is_string($showhidden) )
	{
		$link_list = $modlist = $namelist = array();
		$modlist = list_sections_translated($path);
		foreach ( $modlist as $modl )
		{
			$link = get_section_id($modl['link']);
			$title = $modl['title'];
			$accesskey = $modl['accesskey'];
			if ( $accesskey != "" )
				$accesskey = "accesskey='$accesskey'";
			$tl = "";
			if ( $_FN['lang'] != $_FN['lang_default'] )
				$tl = "&amp;lang={$_FN['lang']}";
			$link_list[] = "<a $accesskey href=\"" . fn_rewritelink("index.php?mod=$link$tl") . "\">$title</a>";
			$namelist[] = $link;
		}
		switch ( $showhidden )
		{
			case "names" :
				//dprint_r($namelist);
				return ($namelist);
				break;
			case "links" :
				return ($link_list);
				break;
		}
	}
	if ( $path != "sections" || $showhidden != false || $recursive != false )
		return list_subsections($path, $showhidden, $recursive);
	$path = "sections";
	static $modlist = null;
	//evito di fare la ricerca tutte le volte per aumentare le prestazioni
	if ( $modlist != null )
	{
		return $modlist;
	}
	$basepath = $path;
	$handle = opendir($path);
	$modlist = array();
	while ( false !== ($file = readdir($handle)) )
	{
		if ( $file != "." and $file != ".." and (!preg_match('/^\./s', $file) and ($file != "CVS")) and is_dir("$path/$file") )
		{
			if ( !preg_match("/^none_/si", $file) || $showhidden === true )
			{
				$modlist[] = str_replace("$basepath/", "", $file);
			}
		}
	}
	closedir($handle);
	fn_natsort($modlist);
	return $modlist;
}
/**
 *
 *
 * Ricava la lista delle sezioni all' interno di una cartella e le traduce
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return array che contiene la lista delle sezioni in coppia link/traduzione, per scorrerlo utilizzare foreach
 */
function list_sections_translated($path = "sections", $showhidden = false, $recursive = false, $forcelang = false)
{
	static $retcache = array();
	if ( isset($retcache[$path]) && $recursive == false && $showhidden == false )
	{
		return $retcache[$path]; //cache per aumentare prestazioni
	}
	$modlist = list_subsections($path, $showhidden, $recursive);
	//$path="sections/$path";
	$ret = array();
	$cs = 1;
	foreach ( $modlist as $mod )
	{
		//forzo l'accesskey per una determinata sezione
		if ( file_exists("$path/$mod/accesskey.php") )
		{
			$coppia = array();
			$t = explode("_", $mod);
			$coppia['prefix'] = isset($t[0]) ? $t[0] : "";
			$tmp = str_replace("none_", "", basename($mod));
			$tmp = preg_replace("/^[0-9][0-9]_/si", "", $tmp);
			$tmp = str_replace("_", " ", $tmp);
			$tmp = getLang("$path/$mod/lang.xml", $tmp, $forcelang);
			$coppia['link'] = str_replace("sections/", "", "$path/$mod");
			$forcekey = file_get_contents("$path/$mod/accesskey.php");
			$coppia['accesskey'] = GetAccesskey($tmp, $coppia['link'], $forcekey);
			$coppia['title'] = $tmp;
			$ret[$cs] = $coppia;
		}
		$cs++;
	}
	$cs = 1;
	foreach ( $modlist as $mod )
	{
		//forzo l'accesskey per una determinata sezione
		if ( !file_exists("$path/$mod/accesskey.php") )
		{
			$tmp = str_replace("none_", "", basename($mod));
			$tmp = preg_replace("/^[0-9][0-9]_/si", "", $tmp);
			$tmp = str_replace("_", " ", $tmp);
			$tmp = getLang("$path/$mod/lang.xml", $tmp, $forcelang);
			$coppia = array();
			$t = explode("_", $mod);
			$coppia['prefix'] = isset($t[0]) ? $t[0] : "";
			$coppia['link'] = str_replace("sections/", "", "$path/$mod");
			$coppia['accesskey'] = GetAccesskey($tmp, $coppia['link']);
			$coppia['title'] = $tmp;
			$ret[$cs] = $coppia;
		}
		$cs++;
	}
	ksort($ret);
	if ( $recursive == false && $showhidden == false )
		$retcache[$path] = $ret;
	return $ret;
}
/**
 * Ricava un accesskey per un link tentando di utilizzare la prima lettera del titolo
 * se e' gia' stata utilizzata per un' altro link tenta con le altre
 * i numeri sono riservati e possono essere utilizzati per le sezioni(es. news)
 * 
 *
 * @param string riferimento al titolo (NON METTERE LE COSTANTI !!!)
 * @param string link
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return accesskey
 */
function GetAccessKey(& $title, $link, $forcekey = "")
{
	global $_FN;
	if ( !isset($_FN['accesskey']) || !is_array($_FN['accesskey']) )
		$_FN['accesskey'] = array();
	$showaccesskey = $_FN['showaccesskey'];
	$titlel = strtolower($title);
	//	dprint_r($accesskey);
	if ( $forcekey != "" )
	{
		if ( $showaccesskey == 1 ) // sottolinea gli accesskey
		{
			$title = "[" . $forcekey . "]$title";
		}
		$_FN['accesskey'][$forcekey] = $link;
		return $forcekey;
	}
	//----------cerco un accesskey libero------------
	for ( $i = 0; $i < strlen($titlel); $i++ )
	{
		$a = $titlel[$i];
		if ( !fn_erg("[a-z]", $a) )
			continue;
		//---------se esiste gia' per quel link esco --------------
		if ( isset($_FN['accesskey'][$a]) && $_FN['accesskey'][$a] == $link )
		{
			if ( $showaccesskey == 1 ) // sottolinea gli accesskey
				$title = "[" . $a . "]&nbsp;$title";
			$_FN['accesskey'][$a] = $link;
			return $a;
		}
		//-----tento con le altre lettere ------
		if ( !isset($_FN['accesskey'][$a]) && !is_numeric($a) )
		{
			$_FN['accesskey'][$a] = $link;
			if ( $showaccesskey == 1 ) // sottolinea gli accesskey
			{
				$title = "[" . $a . "]&nbsp;$title";
			}
			return $a;
		}
	}
	$chrs = array('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
		'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
		',', '.', '-', '+', '\\', '*', '@', '#', '?', '$', '!', '%', '/',
		'(', ')', '=', '^', ';', ':','.', '_', '|');
	foreach ( $chrs as $a )
	{
		if ( !isset($_FN['accesskey'][$a]) )
		{
			if ( $showaccesskey == 1 ) // sottolinea gli accesskey
			{
				$title = "[" . $a . "]$title";
			}
			$_FN['accesskey']["$a"] = $link;
			return $a;
		}
	}
	return "";
}
/**
 * 
 * @param $a
 * @param $b
 */
function fn_natsort_callback($a, $b)
{
	if ( fn_erg("^[0-9]", $a) && fn_erg("^[0-9]", $b) )
	{
		$aa = explode("_", $a);
		$bb = explode("_", $b);
		$aa = $aa[0];
		$bb = $bb[0];
		if ( intval($aa) == intval($bb) )
		{
			return strnatcmp($a, $b);
		}
		return (intval($aa) < intval($bb)) ? -1 : 1;
	}
	return strnatcmp($a, $b);
}
/**
 *
 * @param string $array
 */
function fn_natsort(&$array)
{
	if ( is_array($array) )
		usort($array, "fn_natsort_callback");
}
/**
 * Ricava la lista delle sezioni o sottosezioni all' interno di una cartella
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param string $path dove cercare le sezioni (es. sections/01_pippo)
 * @param bool mostra sezioni nascoste
 * @param bool recursive ricerca ricorsiva nelle sottosessioni
 * @return array che contiene la lista delle sezioni, per scorrerlo utilizzare foreach
 */
function list_subsections($path = "sections", $showhidden = false, $recursive = false, $reset = true)
{
	static $modlist = array();
	static $basepath = null;
	static $cache = array();
	if ( isset($cache[$path . $showhidden . $recursive . $reset]) )
	{
		//dprint_r($cache);
		return $cache[$path . $showhidden . $recursive . $reset];
	}
	while ( strstr($path, '//') )
	{
		$path = str_replace('//', '/', $path);
	}
	if ( $basepath == null || $reset == true )
		$basepath = $path;
	if ( !file_exists($path) || !is_dir($path) )
		return array();
	$handle = opendir($path);
	if ( $recursive == false || $path == "$basepath" ) //se e' la prima chiamata o non e' ricorsiva resetto la lista
		$modlist = array();
	while ( false != ($file = readdir($handle)) )
	{
		if ( $file != "." and $file != ".." and (!preg_match('/^\./s', $file) and ($file != "CVS")) and is_dir("$path/$file") )
		{
			if ( !preg_match("/^none_/si", $file) || $showhidden == true )
			{
				if ( $recursive != false )
				{
					array_push($modlist, str_replace("$basepath/", "", $path . "/" . $file));
					list_subsections($path . "/" . $file, $showhidden, true, false); // se ricorsiva cerco anche le sottosessioni
				}
				else
					array_push($modlist, str_replace("$basepath/", "", $file));
			}
		}
	}
	closedir($handle);
	//dprint_r($modlist);
	//fn_natsort ( $modlist);
	if ( $reset )
	{
		fn_natsort($modlist);
		//dprint_r($modlist);
		$cache[$path . $showhidden . $recursive . $reset] = $modlist;
	}
	return $modlist;
}
/**
 * Ritorna il percorso del file nella versione del tema corrente
 * se questo non ha una versione corrispondente torna il percorso
 * passato come parametro
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string file da temare
 * @return file temato
 */
function fromtheme($file)
{
	global $_FN;
	return file_exists("themes/{$_FN['theme']}/" . $file) ? "themes/{$_FN['theme']}/" . $file : $file;
}
/**
 * Gestione tag per codice html
 *
 * Rimpiazza i pseudo-tag ([b], [i], ecc) con i tag html)
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @author Marco Segato <segatom@users.sourceforge.net> | 20050916: Unificato la funzione per l'uso nel sito e nel forum
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @since 2.5.7
 *
 * @param string $string Stringa da verificare
 * @param string $where Riferimento alla root per l'esecuzione del codice: 'home' per le news, 'forum' per il forum
 * @return string Codice HTML
 */
function tag2html($string)
{
	// verifico provenienza della chiamata e adatto i richiami alle directories
	$string = str_replace("[:)]", "<img src=\"" . fromtheme("images/emoticon/01.png") . "\" alt=\":)\" />", $string);
	$string = str_replace("[:(]", "<img src=\"" . fromtheme("images/emoticon/02.png") . "\" alt=\":(\" />", $string);
	$string = str_replace("[:o]", "<img src=\"" . fromtheme("images/emoticon/03.png") . "\" alt=\":o\" />", $string);
	$string = str_replace("[:p]", "<img src=\"" . fromtheme("images/emoticon/04.png") . "\" alt=\":p\" />", $string);
	$string = str_replace("[:D]", "<img src=\"" . fromtheme("images/emoticon/05.png") . "\" alt=\":D\" />", $string);
	$string = str_replace("[:!]", "<img src=\"" . fromtheme("images/emoticon/06.png") . "\" alt=\":!\" />", $string);
	$string = str_replace("[:O]", "<img src=\"" . fromtheme("images/emoticon/07.png") . "\" alt=\":O\" />", $string);
	$string = str_replace("[8)]", "<img src=\"" . fromtheme("images/emoticon/08.png") . "\" alt=\"8)\" />", $string);
	$string = str_replace("[;)]", "<img src=\"" . fromtheme("images/emoticon/09.png") . "\" alt=\";)\" />", $string);
	// formattazione testo
	$string = str_replace("\n", "<br />", $string);
	$string = str_replace("\r", "", $string);
	$string = str_replace("[b]", "<b>", $string);
	$string = str_replace("[/b]", "</b>", $string);
	$string = str_replace("[i]", "<i>", $string);
	$string = str_replace("[/i]", "</i>", $string);
	$string = str_replace("[quote]", "<blockquote><hr noshade=\"noshade\" /><i>", $string);
	$string = str_replace("[/quote]", "</i><hr noshade=\"noshade\" /></blockquote>", $string);
	$string = str_replace("[code]", "<blockquote><pre>", $string);
	$string = str_replace("[/code]", "</pre></blockquote>", $string);
	//if (isadmin())
	{
		$string = str_replace("[img]", "<br /><img src=\"", $string);
		$string = str_replace("[/img]", "\" alt=\"uploaded_image\" /><br />", $string);
	}
	//$string = preg_replace("/\[youtube\](.+?)\[\/youtube\]/s",'<object width="425" height="355"><param name="movie" value="http://www.youtube.com/v/$1&rel=1"></param><param name="wmode" value="transparent"></param><embed src="http://www.youtube.com/v/$1&rel=1" type="application/x-shockwave-flash" wmode="transparent" width="425" height="355"></embed></object>',$string);
	// colori del testo
	$string = str_replace("[red]", "<font color='ff0000'>", $string);
	$string = str_replace("[green]", "<font color='00ff00'>", $string);
	$string = str_replace("[blue]", "<font color='0000ff'>", $string);
	$string = str_replace("[pink]", "<font color='ff00ff'>", $string);
	$string = str_replace("[yellow]", "<font color='ffff00'>", $string);
	$string = str_replace("[cyan]", "<font color='00ffff'>", $string);
	$string = str_replace("[/red]", "</font>", $string);
	$string = str_replace("[/blue]", "</font>", $string);
	$string = str_replace("[/green]", "</font>", $string);
	$string = str_replace("[/pink]", "</font>", $string);
	$string = str_replace("[/yellow]", "</font>", $string);
	$string = str_replace("[/cyan]", "</font>", $string);
	// WIKIPEDIA
	$items = explode("[/wp]", $string);
	for ( $i = 0; $i < count($items); $i++ )
	{
		$wp = "";
		if ( stristr($items[$i], "[wp") )
		{
			$wp_lang = preg_replace("/.*\\[wp lang=/s", "", $items[$i]);
			$wp_lang = preg_replace("/\\].*/s", "", $wp_lang);
			$wp = preg_replace("/.*\\[wp.*\\]/s", "", $items[$i]);
			$wp = preg_replace("/\\[/wp\\].*/s", "", $wp);
			if ( $wp != "" )
			{
				$nuovowp = "<a style=\"text-decoration: none; border-bottom: 1px dashed; color: blue;\" target=\"new\" href=\"http://$wp_lang.wikipedia.org/wiki/$wp\">$wp</a>";
				$string = str_replace("[wp lang=$wp_lang]" . $wp . "[/wp]", $nuovowp, $string);
			}
		}
	}
	$items = "";
	// URLs
	$items = explode("[/url]", $string);
	for ( $i = 0; $i < count($items); $i++ )
	{
		$url = "";
		if ( stristr($items[$i], "[url]") )
		{
			$url = preg_replace("/.*\\[url\\]/s", "", $items[$i]);
			$url = preg_replace("/\\[\/url\\].*/s", "", $url);
			if ( $url != "" )
			{
				if ( stristr($url, "http://") == FALSE && stristr($url, "https://") == FALSE )
				{
					$nuovourl = "<a target=\"new\" href=\"http://$url\">$url</a>";
				}
				else
				{
					$nuovourl = "<a target=\"new\" href=\"$url\">$url</a>";
				}
				$string = str_replace("[url]" . $url . "[/url]", $nuovourl, $string);
			}
		}
	}
	return ($string);
}
/**
 * printr con il tag PRE, serve per debug
 * @param string $str
 */
function dprint_r($str)
{
	echo "<pre style=\"background-color:#ffffff;color:#000000\" align='left'>";
	print_r($str);
	echo "</pre>";
}
/**
 * 
 * @param scrive l'xml a video $str
 */
function dprint_xml($str)
{
	echo "<pre align='left'>";
	print(htmlspecialchars($str));
	echo "</pre>";
}
/**
 * Elimina gli slash dei post verificando se e' settato magic_quotes_gpc
 * @param string $string
 */
function strippostpslashes($string)
{
	$magic_quotes = (bool) ini_get('magic_quotes_gpc');
	if ( $magic_quotes )
		return stripslashes($string);
	else
		return ($string);
}
/**
 * include i files nella cartella include/javascripts
 */
function IncludeJavascripts()
{
	global $_FN;
	$path_js = "include/javascripts";
	$nocache = "";
	//$nocache = "?".time();
	if ( file_exists($path_js) )
	{
		$dir_js = opendir($path_js);
		$file_js = 0;
		while ( false !== ($filename_js = readdir($dir_js)) )
		{
			$extension_js = null;
			preg_match('/[\.]*[[:alpha:]]+$/s', $filename_js, $extension_js);
			if ( isset($extension_js[0]) && strtolower($extension_js[0]) == ".js" and $filename_js != "." and $filename_js != ".." )
			{
				$array_js[$file_js] = $filename_js;
				$file_js++;
			}
		}
		closedir($dir_js);
		fn_natsort($array_js);
		for ( $i = 0; $i < $file_js; $i++ )
		{
			echo "\n\t<script type=\"text/javascript\" src=\"" . $_FN['siteurl'] . "$path_js/$array_js[$i]$nocache\"></script>";
		}
	}
}
/**
 * include i files css nella cartella include/css
 * @param string $end_tag
 */
function IncludeCss($end_tag = "/")
{
	global $_FN;
	$theme = $_FN['theme'];
	// declaration of all default StyleSheets provided by the system
	$path_css_sys = "include/css";
	if ( file_exists($path_css_sys) )
	{
		$dir_css_sys = opendir($path_css_sys);
		$file_css_sys = 0;
		while ( false !== ($filename_css_sys = readdir($dir_css_sys)) )
		{
			$extension_css_sys = false;
			preg_match('/[\.]*[[:alpha:]]+$/si', $filename_css_sys, $extension_css_sys);
			if ( isset($extension_css_sys[0]) && strtolower($extension_css_sys[0]) == ".css" and $filename_css_sys != "." and $filename_css_sys != ".." and !preg_match("/^none_/si", str_replace('/', '\\/', $filename_css_sys)) )
			{
				$array_css_sys[$file_css_sys] = $filename_css_sys;
				$file_css_sys++;
			}
		}
		closedir($dir_css_sys);
		fn_natsort($array_css_sys);

		for ( $i = 0; $i < $file_css_sys; $i++ )
		{
			//echo "<style>";
			//echo file_get_contents($path_css_sys/$array_css_sys[$i]);
			//echo "</style>";
			$nocache = filemtime("$path_css_sys/{$array_css_sys[$i]}");
			echo "\n\t<link rel='StyleSheet' type='text/css' href='" . $_FN['siteurl'] . "$path_css_sys/$array_css_sys[$i]?$nocache' $end_tag>";
		}
	}
	// declaration of all StyleSheets provided by the theme in use
	$path_css_thm = "themes/$theme";
	if ( file_exists($path_css_thm) )
	{
		$dir_css_thm = opendir($path_css_thm);
		$file_css_thm = 0;
		while ( false !== ($filename_css_thm = readdir($dir_css_thm)) )
		{
			$extension_css_thm = null;
			preg_match('/[\.]*[[:alpha:]]+$/si', $filename_css_thm, $extension_css_thm);
			//dprint_r($extension_css_thm);
			//dprint_r($extension_css_thm);
			//die();
			if ( isset($extension_css_thm[0]) && strtolower($extension_css_thm[0]) == ".css" and $filename_css_thm != "." and $filename_css_thm != ".." and !preg_match("/^none_/si", $filename_css_thm) )
			{
				$array_css_thm[$file_css_thm] = $filename_css_thm;
				$file_css_thm++;
			}
		}
		closedir($dir_css_thm);
		for ( $i = 0; $i < $file_css_thm; $i++ )
		{
			if ( get_file_extension($array_css_thm[$i]) == "css" )
			{
				$nocache = filemtime("$path_css_sys/{$array_css_sys[$i]}");
				echo "\n\t<link rel='StyleSheet' type='text/css' href='" . $_FN['siteurl'] . "$path_css_thm/$array_css_thm[$i]?$nocache' $end_tag>";
			}
		}
	}
}
/**
 * Genera il vackup di un file
 * @param string $file
 */
function backup_file($file)
{
	// genero un file di backup
	global $_FN;
	$time = time();
	while ( file_exists("$file.$time.{$_FN['user']}.bak~") )
		$time++;
	copy($file, "$file.$time.{$_FN['user']}.bak~");
}
/**
 * ricava la data dal numero dato dalla funzione time
 *
 * @param int $postime
 * @return string
 */
function fn_GetDateTime($postime)
{
	global $_FN;
	global $giorni, $mesi;
	$jet_lag = $_FN['jet_lag'];
	$ret = "";
	$ret .= $giorni[date("w", $postime + (3600 * $jet_lag))];
	$ret .= date(" d ", $postime + (3600 * $jet_lag));
	$tmp = date(" m", $postime + (3600 * $jet_lag));
	if ( $tmp < 10 )
		$tmp = str_replace("0", "", $tmp);
	$ret .= $mesi[$tmp - 1];
	$ret .= date(" Y - ", $postime + (3600 * $jet_lag));
	$ret .= date("H:", $postime + (3600 * $jet_lag));
	$ret .= date("i", $postime + (3600 * $jet_lag));
	return $ret;
}
/**
 * Recupera l'icona a partire da un nome di file
 * @param string $filename
 */
function fn_GetIconByFilename($filename)
{
	$ext = get_file_extension($filename);
	$ext = strtolower($ext);
	$dimg = "unknown.png";
	switch ( $ext )
	{
		case "sh" :
			$dimg = "binhex.png";
			break;
		case "xhtml" :
		case "html" :
		case "htm" :
			$dimg = "web.png";
			break;
		case "inc" :
		case "txt" :
		case "xml" :
		case "css" :
		case "" :
			$dimg = "text.png";
			break;
		case "png" :
		case "bmp" :
		case "jpg" :
		case "jpeg" :
		case "ico" :
		case "gif" :
			$dimg = "image.png";
			break;
		case "zip" :
		case "gz" :
			$dimg = "compressed.png";
			break;
		case "mp3" :
		case "wav" :
			$dimg = "sound.png";
			break;
		case "wma" :
		case "mpeg" :
		case "rm" :
			$dimg = "movie.png";
			break;
		default :
			if ( file_exists("images/mime/$ext.png") )
				$dimg = "$ext.png";
			break;
	}
	return fromtheme("images/mime/$dimg");
}
/**
 * Recupera il titolo di una sezione
 * @param string $section
 * @param string $lang
 */
function get_section_title($section, $lang = "")
{
	global $_FN;
	$tmp = preg_replace("/^[0-9]+_|^none_/si", "", basename($section));
	$tmp = str_replace("_", " ", $tmp);
	if ( $lang == "" )
		$lang = $_FN['lang'];
	$title = getLang("sections/$section/lang.xml", $tmp);
	return $title;
}
/**
 * Crea un form di login
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 */
function fn_loginform()
{
	global $_FN;
	$table = new XMLTable("fndatabase", "users", "{$_FN['datadir']}");
	echo "<form method=\"post\" action=\"?mod=" . $_FN['mod'] . "&amp;fnlogin=login\" >";
	if ( $table->primarykey == "email" )
		echo fn_i18n("_FEMAIL");
	else
		echo fn_i18n("_NOMEUTENTE");
	echo "<br /><input type=\"text\" size=\"15\" name=\"username\" /><br />";
	echo _PASSWORD . "<br /><input type=\"password\" size=\"15\" name=\"password\" />";
	echo "&nbsp;<input type=\"submit\" value=\"" . _LOGIN . "\" />";
	echo "</form>";
	$fnlogin = getparam("fnlogin", PAR_GET, SAN_FLAT);
	$fnuser = getparam("username", PAR_POST, SAN_FLAT);
	$fnpwd = getparam("password", PAR_POST, SAN_FLAT);
	if ( $fnlogin == "login" && $fnuser != "" && $fnpwd != "" )
	{
		$lpass = md5($fnpwd);
		$us = get_user($fnuser);
		$passwd = $us['passwd'];
		if ( $passwd != $lpass )
		{
			echo "<br />" . _PASSERR;
		}
	}
}
/**
 * convert newline in correct format
 * 
 * @param string $text
 */
function FN_fixnewline($text)
{
	if ( strtoupper(substr(PHP_OS, 0, 3) == 'WIN') )
	{
		$eol = "\r\n";
	}
	elseif ( strtoupper(substr(PHP_OS, 0, 3) == 'MAC') )
	{
		$eol = "\r";
	}
	else
	{
		$eol = "\n";
	}
	//fix newline
	$text = str_replace("\r\n", "\n", $text);
	$text = str_replace("\r", "", $text);
	$text = str_replace("\n", "$eol", $text);
	return $text;
}
/**
 * url2filepath
 * 
 * @param string theURL
 */
function url2filepath($theURL)
{
	global $WIMPY_BASE, $slash;
	// this sets the sytem / or \ :
	strstr(PHP_OS, "WIN") ? $slash = "\\" : $slash = "/";
	// This is the location of the php file that contains this
	// function. Usually this request is made to files/folders
	// down the directory structure, so the php file that
	// contains these functions is a good "where am i"
	// reference point:
	$WIMPY_BASE['path']['physical'] = getcwd();
	$protocol = "http://";
	if ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on" )
		$protocol = "https://";
	$WIMPY_BASE['path']['www'] = "$protocol" . $_SERVER['HTTP_HOST'];
	$AtheFile = explode("/", $theURL);
	$theFileName = array_pop($AtheFile);
	$AwimpyPathWWW = explode("/", $WIMPY_BASE['path']['www']);
	$AtheFilePath = array_values(array_diff($AtheFile, $AwimpyPathWWW));
	if ( $AtheFilePath )
	{
		$theFilePath = $slash . implode($slash, $AtheFilePath) . $slash . $theFileName;
	}
	else
	{
		$theFilePath = implode($slash, $AtheFilePath) . $slash . $theFileName;
	}
	return ($WIMPY_BASE['path']['physical'] . $theFilePath);
}
/**
 * filepath2url
 * 
 * @author Alessandro Vernassa <speleoalex@gmail.com> 
 * @param string theFilepath
 */
function filepath2url($theFilepath)
{
	global $WIMPY_BASE, $slash;
	// this sets the sytem / or \ :
	strstr(PHP_OS, "WIN") ? $slash = "\\" : $slash = "/";
	// This is the location of the php file that contains this
	// function. Usually this request is made to files/folders
	// down the directory structure, so the php file that
	// contains these functions is a good "where am i"
	// reference point:
	$WIMPY_BASE['path']['physical'] = getcwd();
	$protocol = "http://";
	if ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on" )
		$protocol = "https://";
	$WIMPY_BASE['path']['www'] = "$protocol" . $_SERVER['HTTP_HOST'];
	$AtheFile = explode($slash, $theFilepath);
	$theFileName = array_pop($AtheFile);
	$AwimpyPathFILE = explode($slash, $WIMPY_BASE['path']['physical']);
	$AtheFilePath = array_values(array_diff($AtheFile, $AwimpyPathFILE));
	$thFileURL = implode("/", $AtheFilePath) . "/" . $theFileName;
	return ($WIMPY_BASE['path']['www'] . "$thFileURL");
}
/**
 * converte la codifica dei caratteri
 * @param string $str
 * @param string $charsetFrom
 * @param string $charsetTo
 */
function FN_Convert_Encoding($str, $charsetFrom, $charsetTo)
{
	$str = htmlentities($str, ENT_QUOTES, $charsetFrom);
	$str = html_entity_decode($str, ENT_QUOTES, $charsetTo);
	return $str;
}
/**
 * translate the string
 *
 * @param string $constant
 * @param string $lang
 * @return string
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * 
 */
function fn_i18n($constant, $languageconstant = "")
{
	global $_FN, $_FNMESSAGE;
	static $cache = array();
	$found = true;
	//eval('$text = ' . $constant . ' ;');
	//return $text;
	$constantori = str_replace(" ", "_", $constant);
	$constant = str_replace(" ", "_", $constant);
	$constant = str_replace("!", "_", $constant);
	$constant = str_replace("?", "_", $constant);
	$constant = str_replace("&", "_", $constant);
	$constant = str_replace(",", "_", $constant);
	$constant = str_replace(":", "_", $constant);
	$constant = str_replace(";", "_", $constant);
	if ( isset($cache[$constant]) )
		return $cache[$constant];
	$lang = $_FN['lang'];
	if ( $languageconstant == "" )
		$languageconstant = $lang;
	$suff = "";
	//if ( $_FN['lang'] != $_FN['lang_default'] )
	$suff = "_{$_FN['lang']}";
	//echo "suff=$suff";
	$table = xmldb_form("fndatabase", "fn_i18n", $_FN['datadir'], $lang, "xx," . $_FN['languages']);
	$old = $table->xmltable->GetRecordByPrimaryKey($constantori);
	if ( !isset($old["text$suff"]) || $old["text$suff"] == "" )
	{
		$id = $text = "";
		//echo "constant = $constant;";
		if ( $constant != "" )
		{
			if ( defined($constantori) )
			{

				eval('$text = ' . $constantori . ' ;');
				//echo('$text = ' . $constantori . ' ;');
				//return $text;
			}
			elseif ( isset($_FNMESSAGE[$constantori]) && $_FNMESSAGE[$constantori] != "" )
			{
				$text = $_FNMESSAGE[$constantori];
			}
			else
			{
				$text = str_replace("_", " ", $constant);
				$found = false;
			}
			$values = array("id" => $constantori, "text$suff" => $text);
			if ( $found && !isset($old["text"]) )
			{
				$old = $table->xmltable->InsertRecord($values);
			}
			if ( $found && (!isset($old["text$suff"]) || $old["text$suff"] == "" ) )
			{
				$old = $table->xmltable->UpdateRecord($values);
			}
		}
	}
	else
	{
		if ( isset($old["text$suff"]) && $old["text$suff"] != "" )
			$text = $old["text$suff"];
		else
			$text = str_replace("_", " ", $constant);;
	}
	//dprint_r($old);
	return $text;
}
/**
 * converte una stringa utf-8 in entita'
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 */
class unicode_replace_entities
{
	function UTF8entities($content = "")
	{
		$contents = $this->unicode_string_to_array($content);
		$swap = "";
		$iCount = count($contents);
		for ( $o = 0; $o < $iCount; $o++ )
		{
			$contents[$o] = $this->unicode_entity_replace($contents[$o]);
			$swap .= $contents[$o];
		}
		return mb_convert_encoding($swap, "UTF-8"); //not really necessary, but why not.
	}
	function unicode_string_to_array($string)
	{ //adjwilli
		$strlen = mb_strlen($string);
		while ( $strlen )
		{
			$array[] = mb_substr($string, 0, 1, "UTF-8");
			$string = mb_substr($string, 1, $strlen, "UTF-8");
			$strlen = mb_strlen($string);
		}
		return $array;
	}
	function unicode_entity_replace($c)
	{
		$h = ord($c{0});
		if ( $h <= 0x7F )
		{
			return $c;
		}
		else
		if ( $h < 0xC2 )
		{
			return $c;
		}
		if ( $h <= 0xDF )
		{
			$h = ($h & 0x1F) << 6 | (ord($c{1}) & 0x3F);
			$h = "&#" . $h . ";";
			return $h;
		}
		else
		if ( $h <= 0xEF )
		{
			$h = ($h & 0x0F) << 12 | (ord($c{1}) & 0x3F) << 6 | (ord($c{2}) & 0x3F);
			$h = "&#" . $h . ";";
			return $h;
		}
		else
		if ( $h <= 0xF4 )
		{
			$h = ($h & 0x0F) << 18 | (ord($c{1}) & 0x3F) << 12 | (ord($c{2}) & 0x3F) << 6 | (ord($c{3}) & 0x3F);
			$h = "&#" . $h . ";";
			return $h;
		}
	}
}
/**
 * Converte un link rendendolo coerente con il mod_rewrite
 * 
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param string $href
 * @param string $sep
 */
function fn_rewritelink($href, $sep = "")
{
	global $_FN;
	$modok = false;
	if ( $sep == "" )
	{
		if ( fn_erg("&amp;", $href) )
		{
			$sep = "&amp;";
		}
		else
		{
			$sep = "&";
		}
	}
	if ( $_FN['enable_mod_rewrite'] > 0 )
	{
		///dprint_r($link);
		$urlinfo = parse_url($href);
		$scriptname = isset($urlinfo['path']) ? basename($urlinfo['path']) : "index.php";
		if ( $scriptname != "index.php" )
			return $href;
		else
		{
			$href = $scriptname;
		}
		$var = "";
		if ( isset($urlinfo['query']) )
		{
			$var = str_replace("&amp;", "&", $urlinfo['query']);
		}
		$var = explode('&', $var);
		$arr = array();
		$lang = $_FN['lang'];
		$langid = "";
		if ( $lang != $_FN['lang_default'] )
			$langid = ".$lang";
		$op = "";
		$id = "";
		foreach ( $var as $val )
		{
			$x = explode('=', $val);
			if ( isset($x[1]) && $x[1] != "" )
			{
				if ( strpos($x[1], "/") !== false || strpos($x[1], "-") !== false )
				{
					if ( $x[1] != "" )
						$arr[] = $x[0] . "=" . $x[1];
					else
						$arr[] = $x[0];
				}
				else
				{
					switch ( $x[0] )
					{
						case "lang" :
							$langid = "." . $x[1];
							break;
						case "op" :
							$op = "-" . $x[1];
							break;
						case "id" :
							$id = "-" . $x[1];
							break;
						case "mod" :
							if ( $x[1] != "" )
							{
								$href = $x[1];
								$modok = true;
							}
							break;
						default :
							if ( $x[1] != "" )
								$arr[] = $x[0] . "=" . $x[1];
							else
								$arr[] = $x[0];
							break;
					}
				}
			}
		}
		if ( !$modok )
		{
			if ( $_FN['home_section'] != "" )
			{
				$href = get_section_id($_FN['home_section']);
			}
		}
		$query = implode("$sep", $arr);
		if ( $scriptname == "index.php" )
			$href = "{$href}{$op}{$id}$langid.html";
		if ( $query != "" )
		{
			$href .= "?$query"; // . $urlinfo ['query'];
		}
		//dprint_r($href);
	}
	return $href;
}
/**
 * Elimina ricorsivamente una cartella
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param $dirtodelete cartella da eliminare
 *
 * */
function remove_dir_rec($dirtodelete)
{
	if (!$dirtodelete)
		return;
	if ( strpos($dirtodelete, "../") !== false )
		die(_NONPUOI);
	if ( false !== ($objs = glob($dirtodelete . "/*")) )
	{
		foreach ( $objs as $obj )
		{
			is_dir($obj) ? remove_dir_rec($obj) : unlink($obj);
		}
	}
	rmdir($dirtodelete);
}
/**
 *
 * @global array $_FN
 * @param string $format
 * @return string
 */
function fn_now($format = "Y-m-d H:i:s")
{
	global $_FN;
	return date("Y-m-d H:i:s", time() + (3600 * intval($_FN['jet_lag'])));
}
/**
 *
 * @global array $_FN
 * @return int
 */
function fn_time()
{
	global $_FN;
	return time() + (3600 * intval($_FN['jet_lag']));
}
/**
 *
 * @param array $data
 * @param string $order
 * @param bool $desc
 */
function array_sort_by_key($data, $order, $desc = false)
{
	return xmldb_array_sort_by_key($data, $order, $desc);
}
/**
 *
 * @param string $data
 * @return string
 */
function removexmlcomments($data)
{
	return xmldb_removexmlcomments($data);
}
/**
 *
 * @param compatibilita' con vecchia funzione
 * @param <type> $txt
 */
function fnlog($zone, $txt)
{
	FN_log($txt);
}


function abs_to_local($mybody)
{
	$sdir = "/"; //percorso del file
	$sdir1 = "/"; //percorso del file
	$url = $_FN['siteurl'];
	$sdir = $url . $sdir . "/";
	$mybody = str_replace($sdir, '', $mybody);
	$sub = explode("/", $sdir1);
	$r = "";
	foreach ( $sub as $s )
	{
		if ( $s !== "" )
		{
			$r .= "../";
		}
	}
	$mybody = str_replace($_FN['siteurl'], $r, $mybody);
	return $mybody;
}
/**
 * converte in un file i links a immagini o oggetti da locali ad assoluti
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param string $filename nome del file da convertire
 * @return stringa convertita
 *
 * */
function local_to_abs($filename)
{
	global $_FN;
	if ( !file_exists($filename) )
		return "";
	$sdir = dirname($filename);
	$url = $_FN['siteurl'];
	$sdir = $url . $sdir;
	$file = preg_replace("/^sections\//si", "", $filename);
	//$diff = str_replace ( $url, "", $sdir );
	$ret = '';
	$rows = include_file_to_buffer($filename);
	foreach ( $rows as $line )
	{
		//Proteggo i link esterni dal cambiamento
		$line = preg_replace("/( href| src)='(mms:|skype:|javascript:|\/|https:\/\/|http:\/\/|ftp:\/\/|mailto:|index.php?)/si", "\\1ex='\\2", $line);
		//Proteggo i link esterni dal cambiamento
		$line = preg_replace("/( href| src)=\"(mms:|skype:|javascript:|\/|https:\/\/|http:\/\/|ftp:\/\/|mailto:|index.php?)/si", "\\1ex=\"\\2", $line);
		//Trasformo link ad altre pagine del tipo (es.: href='quella.htm#ciao')
		$line = preg_replace("/ href='([^']+)(\\.jpg|jpeg|bmp|svg|png|swf|gif|txt+)(['|(#.')])/si", " hrefex='$sdir/\\1\\2\\3", $line);
		//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
		$line = preg_replace("/ href=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|png|swf|gif|txt+)([\"|(#.\")])/si", " hrefex=\"$sdir/\\1\\2\\3", $line);
		$mod = getparam("mod", PAR_GET, SAN_FLAT);
		if ( $mod != "modcont" )
		{
			//Trasformo link a file locali (es.: href='mio.tar.gz')
			$line = preg_replace("/ href='([^']+)'/si", " hrefex='$sdir/\\1'", $line);
			//Trasformo link a file locali (es.: href="mio.tar.gz")
			$line = preg_replace("/ href=\"([^\"]+)\"/si", " hrefex=\"$sdir/\\1\"", $line);
			if ( preg_match("/" . str_replace('/', '\\/', $filename) . "/s", "^sections/") )
			{
				//Trasformo link a sezioni dello stesso file (es.: href='#ciao')
				$line = preg_replace("/ href='(#.+)'/si", " hrefex='index.php?mod=$mod&amp;file=$file\\1'", $line);
				//Trasformo link ad altre pagine del tipo (es.: href='quella.htm#ciao')
				$line = preg_replace("/ href='([^']+)(\\.htm|html|php|swf|txt+)(['|(#.')])/si", " hrefex='index.php?mod=$mod&amp;file=\\1\\2\\3", $line);
			}
			else
			{
				//Trasformo link a sezioni dello stesso file (es.: href="#ciao")
				$line = preg_replace("/ href='(#.+)'/si", " hrefex='\\1'", $line);
				//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
				$line = preg_replace("/ href='([^']+)(\\.htm|html|php|swf|txt+)(['|(#.')])/si", " hrefex=$sdir/\\1\\2'", $line);
			}
			if ( preg_match('/' . str_replace('/', '\\/', $filename) . '/si', "^sections") )
			{
				//Trasformo link a sezioni dello stesso file (es.: href="#ciao")
				$line = preg_replace("/ href=\"(#.+)\"/si", " hrefex=\"index.php?mod=$mod&amp;file=$file\\1\"", $line);
				//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
				$line = preg_replace("/ href=\"([^\"]+)(\\.htm|html|php|swf|txt+)([\"|(#.\")])/si", " hrefex=\"index.php?mod=$mod&amp;file=\\1\\2\\3", $line);
			}
			else
			{
				//Trasformo link a sezioni dello stesso file (es.: href="#ciao")
				$line = preg_replace("/ href=\"(#.+)\"/si", " hrefex=\"\\1\"", $line);
				//Trasformo link ad altre pagine del tipo (es.: href="quella.htm#ciao")
				$line = preg_replace("/ href=\"([^\"]+)(\\.htm|html|php|swf|txt+)([\"|(#.\")])/si", " hrefex=$sdir/\\1\\2\"", $line);
			}
		}
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ src='([^']+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[']/si", " src='$sdir/\\1\\2'", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ src=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\"]/si", " src=\"$sdir/\\1\\2\"", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ background='([^']+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[']/si", " background='$sdir/\\1\\2'", $line);
		//Trasformo i percorsi alle immagini locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ background=\"([^\"]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\"]/si", " background=\"$sdir/\\1\\2\"", $line);
		//Trasformo i percorsi alle immagini css locali incluse nella pagina (es.: img src="bella.jpg")
		$line = preg_replace("/ url\\(([^\\)]+)(\\.jpg|jpeg|bmp|svg|swf|png|gif+)[\\)]/si", " url($sdir/\\1\\2)", $line);
		//Ripristino le sorgenti che avevo trasformato e protetto
		$line = preg_replace("/( href| src)ex='/si", "\\1='", $line);
		//Ripristino le sorgenti che avevo trasformato e protetto
		$line = preg_replace("/( href| src)ex=\"/si", "\\1=\"", $line);
		$ret .= $line;
	}
	return $ret;
}
/**
 *
 * @global array $_FN
 * @global array $_FNMESSAGES
 * @param string $filename 
 */
function FN_LoadMessages($filename)
{
	global $_FNMESSAGES;
	if ( !file_exists($filename) )
		return;
	$first = true;
	$handle = fopen("$filename", "r");
	while ( ($data = fgetcsv($handle, 5000, $separator)) !== false )
	{
		$row++;
		if ( $first == true )
		{
			$first = false;
			continue;
		}
		$_FNMESSAGES[$data[0]] = $data[1];
	}
	fclose($handle);
}
?>