<?php
/* **********************************************************************/
/* FlatNux - Flat Text Based Content Management System                  */
/* ============================================                         */
/*                                                                      */
/* Copyright (c) 2004 by Alessandro Vernassa                            */
/* http://flatnux.sourceforge.net                                       */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/* **********************************************************************/
/**
 * File delle funzioni condivise tra il forum e il core di FlatNuke
 *
 * Questo file contiene le procedure di sistema necessarie al funzionamento
 * di Flatnux.
 *
 * @package Flatnux_functions
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * 
 * @version 4.0
 * @copyright Copyright (c) 2003-2005
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License
 */
/**
 * A collection of easy to include PHP functions that sanitize user inputs
 *
 * @author The Open Web Application Security Project - {@link http://www.owasp.org/software/labs/phpfilters.html}
 * @license http://opensource.org/licenses/gpl-license.php GNU General Public License
 */
/**
 * Restituisce un timestamp formattato in secondi
 *
 * @author Marco Segato <segatom@users.sourceforge.net>
 * @since 2.5.7
 *
 * @return number $mtime Timestamp formattato
 */
function get_microtime()
{
	$mtime = microtime();
	$mtime = explode(" ",$mtime);
	$mtime = doubleval($mtime[1]) + doubleval($mtime[0]);
	return $mtime;
}
/**
 * Restituisce la versione attuale di Flatnuke
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return constant Versione di Flatnuke
 */
function get_fn_version($file = "VERSION")
{
	$V = file_get_contents($file);
	$V = str_replace("<?#","",$V);
	$V = str_replace("?>","",$V);
	$V = str_replace("\n","",$V);
	$V = str_replace("\r","",$V);
	if (  !defined("FN_VERSION") )
		define("FN_VERSION",$V);
	return FN_VERSION;
}
/**
 * Elimina i caratteri che possono provocare problemi di sicurezza
 *
 * Elimina dalla stringa passata come parametro i caratteri che possono
 * provocare problemi di sicurezza se eseguiti in concomitanza con codice
 * maligno.
 *
 * @author Marco Segato <segatom@users.sourceforge.net>
 * @since 2.5.7
 *
 * @param string $string Stringa da verificare
 * @return string $string Stringa elaborata
 */
function fnsanitize($string)
{
	$string = str_replace(chr(10),"",$string);
	$string = str_replace(chr(13),"",$string);
	$string = str_replace(chr(00),"",$string);
	$string = str_replace("%00","",$string);
	$string = str_replace("[","&#91;",$string);
	$string = str_replace("]","&#93;",$string);
	if (is_array($string))
		$string="";
	return $string;
}
/**
 * Restituisce un parametro POST o GET
 *
 * Restituisce un parametro controllato con fnsanitize derivante dai
 * metodi POST o GET.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @since 2.5.7
 *
 * @param string $param Nome del parametro
 * @param string $opt Opzione per specificare metodo (POST,GET,ALL, COOKIE, SESSION)
 * @param pointer $sanitize Puntatore a funzione che sanitizza il parametro
 * @return string $param Contenuto del parametro
 */
function getparam($param, $opt, $sanitize)
{
	if (  !isset($sanitize) or ($sanitize == SAN_FLAT) )
		$sanitize = "fnsanitize";
	elseif ( $sanitize == SAN_SYST )
		$sanitize = "sanitize_system_string";
	elseif ( $sanitize == SAN_PARA )
		$sanitize = "sanitize_paranoid_string";
	elseif ( $sanitize == SAN_NULL )
		$sanitize = "sanitize_null_string";
	elseif ( $sanitize == SAN_HTML )
		$sanitize = "sanitize_html_string";
	if ( $opt == PAR_ALL )
	{
		if ( isset($_GET[$param]) )
			return ($sanitize($_GET[$param]));
		else
		{
			if ( isset($_POST[$param]) )
				return ($sanitize($_POST[$param]));
			else
				return ("");
		}
	}
	else 
		if ( $opt == PAR_POST )
		{
			if ( isset($_POST[$param]) )
				return ($sanitize($_POST[$param]));
			else
				return ("");
		}
		else 
			if ( $opt == PAR_GET )
			{
				if ( isset($_GET[$param]) )
					return ($sanitize($_GET[$param]));
				else
					return ("");
			}
			else 
				if ( $opt == PAR_COOKIE )
				{
					if ( isset($_COOKIE[$param]) )
						return ($sanitize($_COOKIE[$param]));
					else
						return ("");
				}
				else 
					if ( $opt == PAR_SESSION )
					{
						if ( isset($_SESSION[$param]) )
							return ($sanitize($_SESSION[$param]));
						else
							return ("");
					}
					else 
						if ( $opt == PAR_SERVER )
						{
							if ( isset($_SERVER[$param]) )
								return ($sanitize($_SERVER[$param]));
							else
								return ("");
						}
						else 
							if ( $opt == PAR_NULL )
							{
								if ( isset($param) )
									return ($sanitize($param));
								else
									return ("");
							}
	return ("");
}
/**
 * fn_lockfile
 * esegue il lock di un file 
 * non funziona su windows !!!!!
 * 
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * 
 * @param string filename
 * non funziona su windows !!!!! 
 * 
 * 
 */
function fn_lockfile($filename)
{
	$file_lock = $filename;
	$fp = fopen($file_lock,"a+");
	$max_retries = 20000;
	if (  !$fp )
	{
		die();
	}
	// keep trying to get a lock as long as possible
	$retries = 0;
	while ( !flock($fp,LOCK_EX + LOCK_NB) and $retries <= $max_retries)
	{
		usleep(rand(1,100));
		$retries++ ;
	}
	return $fp;
}
/**
 * fn_unlockfile
 * non funziona su windows !!!!!
 * 
 * 
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * 
 * @param string filename
 */
function fn_unlockfile($fp)
{
	if (  !flock($fp,LOCK_UN) )
	{
	}
	else
		fclose($fp);
	return;
}
/**
 * Applica un semaforo ad un file
 *
 * il semaforo viene generato tramite un file di testo con estensione .lock
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $user Nome assoluto del file da bloccare
 * @return string ID del semaforo
 */
function fnlock($file_lock)
{
	if ( strtoupper(substr(PHP_OS,0,3) != 'WIN') )
		return fn_lockfile($file_lock);
	$max_retries = 2000;
	$lockfile = $file_lock . ".lock";
	$retries = 0;
	clearstatcache();
	// se non esiste fa subito la open, se la open fallisce, continua il ciclo
	while (false === ($fp = @ fopen($lockfile,"x")))
	{
		if ( $retries >= $max_retries )
		{
			@ unlink($lockfile);
			//fnlog($_FN['vmod'], "||{$_FN['user']}||error lock $lockfile");
			return false;
		}
		usleep(rand(1,20));
		clearstatcache();
		$retries++ ;
	}
	fclose($fp);
	return $lockfile;
}
/**
 * Rilascia un semaforo applicato ad un file
 *
 * Rilascia il semaforo con ID uguale a quello passato come parametro,
 * permettendo nuovamente l'accesso al relativo file.
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $user ID del semaforo
 */
function fnunlock($sem)
{
	if ( strtoupper(substr(PHP_OS,0,3) != 'WIN') )
		return fn_unlockfile($sem);
	@ unlink($sem);
	return;
}
/**
 * Funzione di scrittura controllata di un file
 *
 * Scrive una stringa nel file specificato rispettando una serie di opzioni (facoltative)
 *
 * @author Aldo Boccacci <zorba_@tin.it>
 * @author Alessandro Vernassa
 *
 * @param string $file    nome del file su cui scrivere
 * @param string $text    testo da scrivere
 * @param string $mode    modo di scrittura da passare alla funzione fopen
 * @param array  $options lista delle opzioni di scrittura
 * Le opzioni possono essere:
 * - nonull: la stringa da scrivere non pu essere vuota
 * - nophp:  il testo non pu contenere codice php
 * - nohtml: il testo non pu contenere codice html
 */
function fnwrite($file, $text, $mode = "w", $options = array ())
{
	global $_FN;
	$file = getparam($file,PAR_NULL,SAN_NULL);
	$text = getparam($text,PAR_NULL,SAN_NULL);
	$mode = getparam($mode,PAR_NULL,SAN_NULL);
	$addr = getparam("REMOTE_ADDR",PAR_SERVER,SAN_FLAT);
	clearstatcache();
	//--------controlli ------>
	foreach ( $options as $option )
	{
		if ( trim($option) == "nonull" )
		{
			if ( trim($text,"\t\0 ") == "" )
			{
				FN_log("Writing an empty file is not allowed!");
				return;
			}
		}
		if ( trim($option) == "nophp" )
		{
			if ( preg_match('/\<\?/si',$text) or preg_match('/\?\>/si',$text) )
			{
				FN_log("Writing PHP code is not allowed!");
				return;
			}
		}
		if ( trim($option) == "nohtml" )
		{
			if ( strip_tags($text) != $text )
			{
				FN_log("Writing HTML code is not allowed!");
				return;
			}
		}
	}
	//--------controlli ------<
	$max_retries = 2000;
	$retries = 0;
	$lockfile = $file . ".write.lock";
	clearstatcache();
	// se non esiste fa subito la open, se la open fallisce, continua il ciclo
	while (false === ($fp = @ fopen($lockfile,"x")))
	{
		if ( $retries >= $max_retries )
		{
			@ unlink($lockfile);
			//fnlog($_FN['vmod'], "||{$_FN['user']}||error lock write $lockfile");
			return;
		}
		$sleep = rand(1,20);
		usleep($sleep);
		clearstatcache();
		$retries++ ;
	}
	fwrite($fp,$text);
	fclose($fp);
	clearstatcache();
	//se non  in grado di scrivere esce
	$contents = file_get_contents($lockfile);
	@ unlink($lockfile);
	if ( "$text" != "$contents" )
	{
		//dprint_r("error :$contents");
		return;
	}
	$fp = fopen($file,"wb");
	fwrite($fp,$text);
	fclose($fp);
}
/**
 * fn_copy
 * 
 * 
 */
function fn_copy($s, $d)
{
	$contents = file_get_contents($s);
	$h = fopen($d,"wb");
	if ( $h === false )
		return false;
	fwrite($h,$contents);
	fclose($h);
	if ( $contents != file_get_contents($d) )
	{
		@ unlink($d);
		return false;
	}
	return true;
}
/**
 * Verifica il codice di controllo da FlatNux
 *
 * Verifica che il codice di controllo immesso dall'utente e salvato nel cookie
 * corrisponda a quello del profilo dell'utente stesso.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * 
 * @param string $user Nome assoluto del profilo utente da elaborare
 * @return boolean Vero o falso
 */
function versecid($user)
{
	$mysecid = getparam("secid",PAR_COOKIE,SAN_FLAT);
	if ( $mysecid != "" && getsecid($user) == $mysecid )
		return true;
	else
		return false;
}
/**
 * Verifica il codice di controllo 
 *
 * Verifica che il codice di controllo immesso dall'utente e salvato nel cookie
 * corrisponda a quello del profilo dell'utente stesso.
 * uguale a versecid ma lasciato per retrocompatibilita'
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $user Nome assoluto del profilo utente da elaborare
 * @return boolean Vero o falso
 */
function versecid2($user)
{
	versecid($user);
}
/**
 * Mette un file in una stringa
 *
 * Trasforma il file passato come parametro in una stringa di testo.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $filename Nome relativo alla root del file da elaborare
 * @return string Stringa con il contenuto del file
 */
function get_file($filename, $tentativi = 1)
{
	if (  !file_exists($filename) )
		return array();
	for ( $i = 0;$i < $tentativi;$i++  )
	{
		$string = "";
		$fd = fopen($filename,"r");
		if ( $fd == "" )
			die(_NONPUOI);
		while ( !feof($fd))
		{
			$buffer = fgets($fd,4096);
			$string .= $buffer;
		}
		fclose($fd);
		if ( $string != "" )
			break;
	}
	return $string;
}
/**
 * Restituisce un elemento XML
 *
 * Restituisce un elemento XML da un file passato come parametro.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $elem Nome dell'elemento XML da cercare
 * @param string $xml Nome del file XML da processare
 * @return string Stringa contenente il valore dell'elemento XML
 *
 */
function get_xml_element($elem, $xml)
{
	$buff = preg_replace("/.*<" . $elem . ">/s","",$xml);
	if ( $buff == $xml )
		return false;
	$buff = preg_replace("/<\\/" . $elem . ">.*/s","",$buff);
	return $buff;
}
/**
 * Restituisce l'array di un ramo di elementi XML
 *
 * Restituisce l'array di un ramo di elementi XML da un file passato come parametro.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $elem Nome del ramo di elementi XML da cercare
 * @param string $xml Nome del file XML da processare
 * @return array Array contenente il ramo di elementi XML
 */
function get_xml_array($elem, $xml)
{
	$buff = explode("</" . $elem . ">",$xml);
	array_splice($buff,count($buff) - 1);
	return $buff;
}
/**
 * Restituisce il livello di un utente
 *
 * Restituisce il livello dell'utente passato come primo parametro; il secondo
 * parametro serve per gestire il riferimento alla cartella dove sono
 * presenti gli utenti, a seconda che la chiamata parta dall'homepage oppure
 * dal forum:
 * - 0 per un utente registrato;
 * - da 1 a 9 per un utente registrato di livello intermedio;
 * - 10 per un utente amministratore.
 *
 * @author Simone Vellei <simone_vellei@users.sourceforge.net>
 *
 * @param string $admin Nome assoluto del profilo utente da elaborare
 * @return string Livello dell'utente
 */
function getlevel($admin)
{
	$admin = get_user($admin);
	$level = isset($admin['level']) ? $admin['level'] :  -1;
	return ($level);
}
/**
 * Elimina ricorsivamente una cartella
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @param $dirtodelete cartella da eliminare
 *
 **/
function rmdird($dirtodelete)
{
	if ( strstr($dirtodelete,"../") )
		die(_NONPUOI);
	if ( false != ($objs = glob($dirtodelete . "/.*")) )
	{
		foreach ( $objs as $obj )
		{
			if (  !is_dir($obj) )
				unlink($obj);
			else
			{
				if ( basename($obj) != "." && basename($obj) != ".." )
				{
					rmdird($obj);
				}
			}
		}
	}
	if ( false != ($objs = glob($dirtodelete . "/*")) )
	{
		foreach ( $objs as $obj )
		{
			is_dir($obj) ? rmdird($obj) : unlink($obj);
		}
	}
	rmdir($dirtodelete);
}
/**
 * muove una cartella dentro un altra
 * 
 */
function fn_movedir($fromDir, $toDir, $verbose = false)
{
	//verifico che sia scrivibile
	$dirs = list_subsections($fromDir,true,true);
	foreach ( $dirs as $dir )
	{
		if (  !is_writable("$fromDir/$dir") )
		{
			if ( $verbose )
				echo "<br />$fromDir/$dir not writable";
			return false;
		}
		$handlesource = opendir("$fromDir/$dir");
		while (false !== ($file_hs = readdir($handlesource)))
		{
			if (  !is_dir("$fromDir/$dir/$file_hs") && $file_hs != "." && $file_hs != "..." )
			{
				if (  !is_writable("$fromDir/$dir/$file_hs") )
				{
					if ( $verbose )
						echo "<br />$fromDir/$dir/$file_hs not writable";
					return false;
				}
			}
		}
	}
	//se riesco a acopiare allora elimino il sorgente
	if ( fn_copydir($fromDir,$toDir,1) )
		rmdird($fromDir);
	return true;
	//die();
}
/**
 * copia una cartella dentro un altra
 * 
 */
function fn_copydir($fromDir, $toDir, $verbose = false)
{
	$dirs = list_subsections($fromDir,true,true);
	if ( $verbose )
		dprint_r("$fromDir => $toDir");
	if (  !is_dir($toDir) )
		return false;
	if (  !is_writable($toDir) )
		return false;
	if ( is_dir($fromDir) && is_dir($toDir) )
	{
		if (  !is_writable($toDir) )
			return false;
		if ( file_exists($toDir . "/" . basename($fromDir)) )
		{
			if ( $verbose )
				echo "<br />" . $toDir . "/" . basename($fromDir) . " exists";
			return false;
		}
		if (  !mkdir($toDir . "/" . basename($fromDir)) )
			return false;
		if ( $verbose )
			echo 'Directory created: ' . $toDir;
		$toDir = $toDir . "/" . basename($fromDir);
	}
	$handlesource = opendir("$fromDir");
	while (false !== ($file_hs = readdir($handlesource)))
	{
		if (  !is_dir("$fromDir/$file_hs") && $file_hs != "." && $file_hs != "..." )
		{
			if ( $verbose )
				echo "<br /><b>copy file $fromDir/$file_hs => $toDir/$file_hs</b>";
			if (  !copy("$fromDir/$file_hs","$toDir/$file_hs") )
				return false;
		}
	}
	foreach ( $dirs as $dir )
	{
		mkdir("$toDir/$dir");
		if ( $verbose )
			echo "<br /><pre>mkdir: $toDir/$dir</pre>";
		$handlesource = opendir("$fromDir/$dir");
		while (false !== ($file_hs = readdir($handlesource)))
		{
			if (  !is_dir("$fromDir/$dir/$file_hs") && $file_hs != "." && $file_hs != "..." )
			{
				if ( $verbose )
					echo "<br /><b>copy file $fromDir/$dir/$file_hs => $toDir/$dir/$file_hs</b>";
				if (  !copy("$fromDir/$dir/$file_hs","$toDir/$dir/$file_hs") )
					return false;
			}
		}
	}
	return true;
}
/**
 * identifica una sezione qualunque sia la sua posizione
 * per funzionare all' interno della sessione occorre mettere
 * un file che si chiama NOME_SESSIONE.section
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 * @param string $section percorso della sessione
 * @return il percorso del file in cui si trova la sessione
 **/
function sectionlocation($sectionid)
{
	if ( $sectionid == "" )
		return false;
		//se la sezione esiste torna il path (qui non serve la cache)
	if ( file_exists("sections/$sectionid") )
	{
		return $sectionid;
	}
	static $cache = array();
	//elimino le doppie barre
	while (strstr($sectionid,"//") !== false)
		$sectionid = str_replace("//","/",$sectionid);
		//cache per aumentare le prestazioni
	if ( isset($cache[$sectionid]) )
		return $cache[$sectionid];
	$namepath = explode("/",$sectionid);
	$filepath = "";
	$exists = false;
	foreach ( $namepath as $dir )
	{
		$exists = false;
		if ( file_exists("sections/$filepath" . "$dir") )
		{
			$filepath .= "$dir/";
			$exists = true;
		}
		else
		{
			$exists = false;
			$handle_dir = opendir("sections/$filepath");
			//echo "<br /> apro : sections/$filepath";
			while (false !== ($file = readdir($handle_dir)))
			{
				if ( ($file != ".") && ($file != "..") && is_dir("sections/$filepath/$file") && file_exists("sections/$filepath/$file/$dir.section") )
				{
					$filepath .= "$file/";
					$exists = true;
					break;
				}
			}
		}
	}
	if ( ($len = strlen($filepath)) > 1 )
		$filepath = substr($filepath,0,$len - 1);
	if ( $exists == false )
		return false;
	$cache[$sectionid] = $filepath;
	return $filepath;
}
/**
 * Torna a true se la pagina  stata visitata da uno spider
 * la lista degli spider si trova in conf/spiders.conf
 * @return bool true o false
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 */
function is_spider()
{
	global $_FN;
	if ( file_exists("{$_FN['datadir']}/listspiders.conf") && isset($_SERVER['HTTP_USER_AGENT']) )
	{
		$handle = fopen("{$_FN['datadir']}/listspiders.conf","r");
		while ( !feof($handle))
		{
			$buffer = fgets($handle,4096);
			if ( stristr($_SERVER['HTTP_USER_AGENT'],$buffer) )
			{
				fclose($handle);
				return true;
			}
		}
		fclose($handle);
	}
	return false;
}
/**
 * Controllo sulla validit di una stringa alfanumerica
 *
 * Esegue un controllo sulla stringa passata come parametro, verificando che
 * contenga unicamente caratteri oppure numeri, secondo lo standard UNIX.
 *
 * @author Marco Segato <segatom@users.sourceforge.net>
 * @since 2.5.7
 *
 * @param string $string Stringa da verificare
 * @return boolean Vero o Falso
 */
function is_alphanumeric($string)
{
	if ( preg_match("/^[a-z0-9_]+$/si",$string) )
		return (true);
	else
		return (false);
}
function is_alphanumericdot($string)
{
	if ( preg_match("/^[a-z0-9_\.]+$/si",$string) )
		return (true);
	else
		return (false);
}


/**
 * Restituisce l'estensione di un file
 *
 * Restituisce l'estensione di un file passato come parametro.
 *
 * @author Marco Segato <segatom@users.sourceforge.net>
 * @since 2.5.7
 *
 * @param string $string Nome del file da verificare
 * @return string $extension Estensione del file
 */
function get_file_extension($filename)
{
	$filename = getparam($filename,PAR_NULL,SAN_FLAT);
	if (  !strstr($filename,".") )
		return "";
	$tmp = explode(".",$filename);
	$extension = $tmp[count($tmp) - 1];
	return $extension;
}
/**
 * Restistuisce TRUE se l'utente collegato pu vedere la sezione specificata da $mod
 *
 * Questa funzione restituisce TRUE se l'utente collegato al portale ha i permessi per visualizzare la sezione
 * richiesta.
 * Indicando anche il secondo parametro si possono verificare i permessi di un determinato utente.
 *
 * @author Alessandro Vernassa<speleoalex@gmail.com>
 *
 * @param string $mod  il mod della sezione di cui verificare i permessi di visione
 * @param string $user l'utente di cui verificare i permessi. Se non viene specificato questo parametro,
 * o se la stringa  vuota si verifica l'utente attualmente collegato al portale
 * @return TRUE se l'utente pu vedere la sezione, FALSE in caso contrario
 */
function user_can_view_section($mod, $user = "")
{
	global $_FN;
	$mod = getparam($mod,PAR_NULL,SAN_FLAT);
	if (  !check_path($mod,"","false") )
		return false;
	if ( $user == "" )
	{
		$user = $_FN['user'];
	}
	else
	{
		$user = getparam($user,PAR_NULL,SAN_FLAT);
	}
	if (  !is_dir("sections/$mod") )
		return false;
	$perms = getsectperm("sections/$mod");
	//dprint_r($perms);
	$sectlevel = $perms['level'];
	$userlevel = getlevel($user);
	
	//dprint_r($perms);
	if ( $userlevel >= 10 )
		return true;
	
	if ( $sectlevel > $userlevel )
		return false;
	if ( $user!="" && $user == $perms['user_view'] )
		return true;
	
	if ( $user!="" &&  user_in_group($user,$perms['group_view']) )
		return true;
	if ( $perms['user_view'] != "" )
		return false;
	if ( $perms['group_view'] != "" )
		return false;
	
	return true;
}
/**
 * Restistuisce TRUE se l'utente collegato pu vedere la sezione specificata da $mod
 *
 * Questa funzione verifica se l'utente collegato ha i permessi di scrittura nella sezione
 * passata come parametro.
 * Indicando anche il secondo parametro si possono verificare i permessi di un determinato utente.
 * La funzione non controlla i permessi di scrittura sui file a livello di filesystem: questi andranno
 * controllati a parte.
 *
 * @author Alessandro Vernassa<speleoalex@gmail.com>
 *
 * @param string $mod  il mod della sezione di cui verificare i permessi di scrittura
 * @param string $user l'utente di cui verificare i permessi. Se non viene specificato questo parametro,
 * o se la stringa  vuota si verifica l'utente attualmente collegato al portale
 * @return TRUE se l'utente pu modificare la sezione, FALSE in caso contrario
 */
function user_can_edit_section($mod, $user = "")
{
	global $_FNUSER;
	$mod = getparam($mod,PAR_NULL,SAN_FLAT);
	if (  !check_path($mod,"","false") )
		return FALSE;
	if ( $user == "" )
	{
		$user = $_FNUSER;
	}
	else
		$user = getparam($user,PAR_NULL,SAN_FLAT);
	if (  !is_dir("sections/$mod") )
		return FALSE;
	return can_modify($user,"sections/$mod");
}
/**
 * torna a true se l'utente  autorizzato alla modifica della sezione
 * verifica i permessi per utente/gruppo per la sezione
 *
 * @param string username
 * @param string section
 * @return bool
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 **/
function can_modify($username, $folder)
{
	if ( getlevel($username) >= 10 )
	{
		return true; //amministratore
	}
	//dprint_r($folder);
	$sectperm = getsectperm($folder);
	if ( $username == $sectperm['user'] && $sectperm['user'] != "" ) //appartiene all' utente
		return true;
	$uservalues = get_user($username);
	if (isset($uservalues['group']))
	{
		$usergroups = explode(",",$uservalues['group']);
		if ( in_array($sectperm['group'],$usergroups) && $sectperm['group'] != "" )
			return true; //appartiene al gruppo
	}
	return false;
}
/**
 * can_modify
 *
 * torna a true se l'utente  autorizzato alla modifica del file
 * @param string username
 * @param string filename
 * @return bool
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 *
 **/
function can_modify_file($username, $filename)
{
	if ( is_admin() )
	{
		return true; //amministratore pu modificare tutto
	}
	if ( stristr(get_file_extension($filename),"php") ) //non posso modificare files php
		return false;
	if ( stristr(get_file_extension($filename),"php3") ) //non posso modificare files php
		return false;
	if ( stristr(get_file_extension($filename),"php4") ) //non posso modificare files php
		return false;
	if ( stristr(get_file_extension($filename),"php5") ) //non posso modificare files php
		return false;
	if ( stristr(get_file_extension($filename),"inc") ) //non posso modificare files php
		return false;
	if ( stristr(get_file_extension($filename),"bak~") ) //non posso modificare files di backup
		return false;
	$t = basename($filename);
	if ( isset($t[0]) && $t[0] == "." )
		return false;
	if ( stristr(get_file_extension($filename),"bak~") ) //non posso modificare files di backup
	{
		return false;
	}
	if ( stristr(get_file_extension($filename),"inc") ) //non posso modificare files php
		return false;
	$dirname = realpath(DirName($filename));
	//$pathfn=substr($_SERVER['SCRIPT_FILENAME'], 0, (strrpos($_SERVER['SCRIPT_FILENAME'], "/")))."/sections";
	//	echo "$dirname=$pathfn";
	//$section = str_replace("$pathfn", "", $dirname);
	return can_modify($username,$dirname);
}
/**
 * Ritorna true se l'utente ha il livello 10
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return true/false
 */
function isadmin()
{
	$user = getparam("myforum",PAR_COOKIE,SAN_FLAT);
	if ( $user != "" )
	{
		if ( (getlevel($user) == 10) && versecid($user) )
		{
			return true;
		}
	}
	return false;
}
/**
 * Ritorna true se l'utente ha il livello 10
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return true/false
 */
function is_admin()
{
	return isadmin();
}
/**
 * Ritorna la lista dei gruppi
 *
 * @author Alessandro Vernassa <speleoalex@gmail.com>
 * @return array che contiene la lista dei gruppi
 */
function get_grouplist()
{
	global $_FN;
	$groups = array();
	$table = new XMLTable("fndatabase","groups","{$_FN['datadir']}");
	$grouplist = $table->GetRecords();
	foreach ( $grouplist as $g )
		$groups[] = $g['groupname'];
	return $groups;
}

/**
 * verifica se una stringa contiene spam
 * @param string stringa da controllare
 * @param string contesto da verificare
 *
 * @return torna a true se la stringa contiene dello spam
 **/
function is_spam($string, $context = "spam")
{
	global $_FN;
	if (  !file_exists("{$_FN['datadir']}/spam/$context.php") )
		return false;
	//es. ([^[:alpha:]]|^)t([^[:alpha:]]*)e([^[:alpha:]]*)s([^[:alpha:]]*)t([^[:alpha:]]|$)
	// considera spam "test" oppure t e s t oppure t.e.s.t. e non testare
	//es. ([^[:alpha:]]|^)test([^[:alpha:]]|$)
	// considera spam "test" oppure  oppure "test" e non testare
	$spams = file("{$_FN['datadir']}/spam/$context.php");
	$l = count($spams);
	for ( $i = 1;$i < $l;$i++  )
	{
		$spamrow = $spams[$i];
		$spamrow = str_replace("\n","",str_replace("\r","",$spamrow));
		if (trim(ltrim($spamrow))=="")
			continue;
		if ( $string != "" && $spamrow != "" && preg_match("/" . str_replace("/","\\/",$spamrow) . "/si",$string) )
		{
			return true;
		}
	}
	return false;
}
/**
 * elimina i \n e i \r da una stringa
 * @param string string
 * 
 */
function clearnr($string)
{
	return str_replace("\r","",str_replace("\n","",$string));
}
/**
 * Verifica se  stato inserito un campo di E-mail semanticamente corretto
 * 
 * @param string email
 * 
 */
function checkmail($email)
{
	if ( preg_match('/^([a-z0-9_\.-])+@(([a-z0-9_-])+\.)+[a-z]{2,6}$/si',trim($email)) )
		return true;
	else
		return false;
}
/**
 * verifica che un file sia un' immagine
 *
 * @param string $imagepath
 * @return bool
 */
function is_image($imagepath)
{
	if ( @ getimagesize($imagepath) == false )
		return false;
	return true;
}
/**
 * Rende un link W3C compliant
 *
 * @author Aldo Boccacci <zorba_@tin.it>
 *
 * @param string $path URL da codificare
 * @return string URL codificato
 */
function rawurlencodepath($path)
{
	$parts = array();
	$parts = explode('/',$path);
	for ( $i = 0;$i < count($parts);$i++  )
	{
		$parts[$i] = rawurlencode($parts[$i]);
	}
	return implode('/',$parts);
}
/**
 * Controlla il percorso di file che gli viene passato come parametro.
 *
 * Questa funzione verifica la validit del percorso passato come parametro. Innanzitutto viene
 * verificata la presenza di caratteri potenzialmente pericolosi.
 * Con il secondo parametro si indica eventualmente la cartella all'interno della quale deve trovarsi il file
 * (o sue sottocartelle).
 * Con il terzo parametro si indica se il file pu avere estensione "php*"
 * Se il percorso rispetta tutte le condizioni la funzione restituisce TRUE,
 * in caso contrario restituisce FALSE.
 *
 * @author Aldo Boccacci <zorba_@tin.it>
 * @since 2.5.8
 *
 * @param string $filename il nome del file
 * @param string $dirbase il file deve essere all'interno di questa directory o di una sua sottodirectory
 * @param string $allow_php se settato a "true" permette che il file abbia estensione .php e simili
 */
function check_path($filename, $dirbase, $allow_php)
{
	if ( stristr($filename,"..") )
		return FALSE;
	if ( stristr($filename,"%00") )
		return FALSE;
	if ( stristr($filename,chr(13)) )
		return FALSE;
	if ( stristr($filename,chr(10)) )
		return FALSE;
	if ( stristr($filename,chr(00)) )
		return FALSE;
	if ( stristr($filename,"://") )
		return FALSE;
	if ( stristr($filename,"?") )
		return FALSE;
	if ( stristr($filename,"&") )
		return FALSE;
	if ( stristr($filename,"$") )
		return FALSE;
	if ( stristr($filename,"[") )
		return FALSE;
	if ( stristr($filename,"]") )
		return FALSE;
	if ( stristr($filename,"(") )
		return FALSE;
	if ( stristr($filename,")") )
		return FALSE;
	if ( stristr($filename,"<") )
		return FALSE;
	if ( stristr($filename,">") )
		return FALSE;
	if ( $dirbase != "" )
	{
		$path = $filename;
		$limit = 0;
		//controlla / iniziale
		while (preg_match("/^\//si",$path))
		{
			$path = preg_replace("/^\//si","",$path);
			if ( $limit == 5 )
				return FALSE;
			$limit++ ;
		}
		if (  !preg_match("/^$dirbase/si",$path) )
			return FALSE;
		if ( preg_match("/^forum/si",$path) )
			return FALSE;
	}
	if ( $allow_php != "true" )
	{
		if ( preg_match('/\.php.$|\.php$/si',$filename) )
			return FALSE;
	}
	return TRUE;
}
/**
 * Crea tutte le tabelle di sistema
 * 
 */
function InitFNTables()
{
	global $_FN;
	if (  !file_exists("{$_FN['datadir']}/fndatabase") )
	{
		echo createxmldatabase("fndatabase","{$_FN['datadir']}");
	}
	//----------------utenti------------------
	if (  !file_exists("{$_FN['datadir']}/fndatabase/fn_i18n.php") )
	{
		$fields = array();
		//usename -non eliminare
		$fields[1]['name'] = "id";
		$fields[1]['frm_required'] = 1;
		$fields[1]['primarykey'] = "1";
		$fields[1]['size'] = "128";
		$fields[2]['name'] = "text";
		$fields[2]['frm_cols'] = "60";
		$fields[2]['frm_rows'] = "4";
		$fields[2]['type'] = "text";
		$fields[2]['frm_multilanguages'] = "auto";
		echo createxmltable("fndatabase","fn_i18n",$fields,"{$_FN['datadir']}","i18n");
	}
	//----------------utenti------------------
	if (  !file_exists("{$_FN['datadir']}/fndatabase/users.php") )
	{
		$fields = array();
		//usename -non eliminare
		$fields[1]['name'] = "username";
		$fields[1]['frm_required'] = 1;
		$fields[1]['frm_it'] = "Username";
		$fields[1]['frm_en'] = "Username";
		$fields[1]['frm_es'] = "Username";
		$fields[1]['frm_de'] = "Username";
		$fields[1]['frm_validator'] = "is_alphanumeric";
		$fields[1]['frm_allowupdate'] = "0";
		$fields[1]['primarykey'] = "1";
		$fields[1]['size'] = "128";
		//password -non eliminare
		$fields[2]['name'] = "passwd";
		$fields[2]['frm_it'] = "Password";
		$fields[2]['frm_en'] = "Password";
		$fields[2]['frm_es'] = "Password";
		$fields[2]['frm_de'] = "Password";
		$fields[2]['frm_type'] = "password";
		$fields[2]['frm_retype'] = "1";
		$fields[2]['frm_required'] = "1";
		//email -non eliminare
		$fields[3]['name'] = "email";
		$fields[3]['frm_it'] = "E-mail";
		$fields[3]['frm_en'] = "E-mail";
		$fields[3]['frm_es'] = "E-mail";
		$fields[3]['frm_de'] = "E-mail";
		$fields[3]['frm_fr'] = "E-mail";
		$fields[3]['frm_validator'] = "checkmail";
		$fields[3]['showinprofile'] = "1";
		$fields[3]['frm_required'] = "1";
		$fields[3]['frm_allowupdate'] = "onlyadmin";
		//email visibile
		$fields[4]['name'] = "emailhidden";
		$fields[4]['type'] = "check";
		$fields[4]['frm_it'] = "Nascondi email ad altri utenti";
		$fields[4]['frm_en'] = "Hide email to other users";
		$fields[4]['frm_es'] = "Ocultar email a otros usuarios";
		$fields[4]['frm_de'] = "Hide E-Mail an andere Benutzer";
		$fields[4]['frm_fr'] = "Hide email  d'autres utilisateurs";
		$fields[4]['frm_help_it'] = "La tua email non sar&agrave; visibile ad altri utenti";
		$fields[4]['showinprofile'] = "0";
		$fields[4]['frm_required'] = "0";
		$fields[4]['frm_show'] = "1";
		//nome
		$fields[5]['name'] = "name";
		$fields[5]['frm_it'] = "Nome";
		$fields[5]['frm_help_it'] = "Immetti il tuo nome qui";
		$fields[5]['frm_en'] = "Name";
		$fields[5]['frm_es'] = "Nombre";
		$fields[5]['frm_de'] = "Name";
		$fields[5]['frm_fr'] = "Nom";
		$fields[5]['showinprofile'] = "1";
		$fields[5]['frm_help_en'] = "Insert your name here";
		//link
		$fields[6]['name'] = "link";
		$fields[6]['frm_it'] = "Link";
		$fields[6]['frm_en'] = "Link";
		$fields[6]['frm_es'] = "Link";
		$fields[6]['frm_de'] = "Link";
		$fields[6]['frm_help_it'] = "URL del tuo sito";
		$fields[6]['frm_default'] = "http://";
		$fields[6]['showinprofile'] = "1";
		//avatar tra i predefiniti
		$fields[7]['name'] = "avatarimage";
		$fields[7]['type'] = "varchar";
		$fields[7]['frm_it'] = "Avatar predefiniti";
		$fields[7]['frm_en'] = "Avatar";
		$fields[7]['frm_es'] = "Avatar";
		$fields[7]['frm_de'] = "Avatar";
		$fields[7]['frm_help_it'] = "Qui puoi sceglere una piccola immagine che ti identifichi tra quelle predefinite";
		$fields[7]['foreignkey'] = "avatars";
		$fields[7]['fk_link_field'] = "filename";
		$fields[7]['fk_show_field'] = "filename";
		$fields[7]['frm_show_image'] = "filename";
		//avatar personalizzato
		$fields[8]['name'] = "avatar";
		$fields[8]['type'] = "image";
		$fields[8]['frm_it'] = "Carica un avatar personalizzato";
		$fields[8]['frm_en'] = "Upload custom avatar";
		$fields[8]['frm_es'] = "Upload custom avatar";
		$fields[8]['frm_de'] = "Upload custom avatar";
		$fields[8]['frm_help_it'] = "Qui puoi uplodare una piccola immagine (massimo 80x80) che ti identifichi";
		$fields[8]['frm_maximagesize'] = "100";
		$fields[8]['thumbsize'] = "24";
		//job
		$fields[9]['name'] = "job";
		$fields[9]['frm_it'] = "Professione";
		$fields[9]['frm_en'] = "Job";
		$fields[9]['frm_es'] = "Profesin";
		$fields[9]['frm_de'] = "Beruf";
		$fields[9]['showinprofile'] = "1";
		//provenienza
		$fields[10]['name'] = "prov";
		$fields[10]['frm_it'] = "Provenienza";
		$fields[10]['frm_en'] = "Provenience";
		$fields[10]['frm_es'] = "Proveniencia";
		$fields[10]['frm_de'] = "Bundesland";
		//firma
		$fields[11]['name'] = "signature";
		$fields[11]['type'] = "text";
		$fields[11]['frm_it'] = "Firma";
		$fields[11]['frm_en'] = "Signature";
		$fields[11]['frm_es'] = "Firma";
		$fields[11]['frm_de'] = "Signatur";
		//condizioni
		$fields[12]['name'] = "conditions";
		$fields[12]['type'] = "conditions_check";
		$fields[12]['frm_it'] = "Condizioni";
		$fields[12]['frm_en'] = "Conditions";
		$fields[12]['frm_es'] = "Condiciones";
		$fields[12]['frm_de'] = "Bedingungen";
		$fields[12]['frm_fr'] = "Conditions";
		$fields[12]['frm_validator'] = "fn_accept_condition_validator";
		$fields[12]['frm_allowupdate'] = "0";
		//-da qui in poi non eliminare
		//livello
		$fields[13]['name'] = "level";
		$fields[13]['defaultvalue'] = "0";
		$fields[13]['frm_show'] = "0";
		//gruppo
		$fields[14]['name'] = "group";
		$fields[14]['frm_show'] = "0";
		//ip
		$fields[15]['name'] = "ip";
		$fields[15]['frm_show'] = "0";
		//numero per registrazione
		$fields[16]['name'] = "rnd";
		$fields[16]['frm_show'] = "0";
		//utente attivo
		$fields[17]['name'] = "active";
		$fields[17]['frm_it'] = "Utente attivo";
		$fields[17]['frm_en'] = "Active user";
		$fields[17]['frm_es'] = "Active user";
		$fields[17]['frm_de'] = "Active user";
		$fields[17]['frm_help_it'] = "Se hai abilitato la registrazione tramite email l&quot; utente non &egrave; attivo fino al completamento della registrazione, puoi forzare l&quot; attivazione qui";
		$fields[17]['type'] = "check";
		$fields[17]['frm_setonlyadmin'] = "1";
		$fields[17]['frm_show'] = "1";
		echo createxmltable("fndatabase","users",$fields,"{$_FN['datadir']}");
	}
	if (  !file_exists("{$_FN['datadir']}/fndatabase/groups.php") )
	{
		$fields = array();
		$fields[1]['name'] = "groupname";
		$fields[1]['primarykey'] = 1;
		$fields[1]['frm_required'] = 1;
		$fields[1]['frm_validator'] = "is_alphanumeric";
		echo createxmltable("fndatabase","groups",$fields,"{$_FN['datadir']}");
		$r['groupname'] = 'users';
		$table = new XMLTable("fndatabase","groups","{$_FN['datadir']}");
		$table->InsertRecord($r);
	}
	if (  !file_exists("{$_FN['datadir']}/spam") )
		mkdir("{$_FN['datadir']}/spam");
	if (  !file_exists("{$_FN['datadir']}/spam/spam.php") )
	{
		$spam = "<?php exit(0) /* regular expressions */?>
http://votvot.vo.funpic.org/index-lotto-649.html
http://malinki.ma.funpic.org/index-pontiac-solstice.html
http://btramadol.coz
http://buyvalium.coz
http://air-buy-cheap-ticket.blogspot.com
http://krasavcheg.xf.cz
([^[:alpha:]]|^)http://adipexonline.web.fc2.com([^[:alpha:]]|^)
([^[:alpha:]]|^)planet.nana.co.il([^[:alpha:]]|^)
([^[:alpha:]]|^)phentermine([^[:alpha:]]|^)
http://www.bestfdameds.com
http://www.fdarxmeds.com
http://www.npmeds.com
http://www.mailrxmeds.com
http://www.rxmedslist.com
http://www.fdamedstore.com
http://www.npdpharmacy.com
http://www.rxmeds4you.net
http://www.storerxmeds.com
http://purchase-xanax-online.daj.pl
([^[:alpha:]]|^)viagra([^[:alpha:]]|^)
http://mdshorty.diaryland.com/buyvalium.html
http://home.earthlink.net/~hydrocodone-online
http://tramadol-online.envy.nu
http://ativan.q.fm
http://atib.prv.pl
http://www.fdarxpharmacy.com
http://www.nppharmacy.com
http://www.spheresofexchange.org
http://aristide.uv.ro
http://ebay.myadsmedia.com
http://hmciran.com
http://www.worldcitytourist.com
http://raiandcompany.com
http://www.gsmtools.com.ua
http://munichilca.gob.pe
http://balajijmkt.com
http://africanhuntermagazines.com
http://soosoocandle.com
		";
		$fp = fopen("{$_FN['datadir']}/spam/spam.php","w");
		fwrite($fp,$spam);
		fclose($fp);
	}
}
/**
 * copies everything from directory $fromDir to directory $toDir
 * and sets up files mode $chmod
 * 
 * @author: Anton Makarenko
 * 
 * 
 **/
/**
 * sample usage:
 * 
 * WARNING:
 * if You set wrong $chmod then You'll not be able to access files and directories
 * in destination directory.
 * For example: once upon a time I've called the function with parameters:
 * copydir($fromDir,$toDir,true);
 * What happened? I've forgotten one parameter (chmod)
 * What happened next? Those files and directories became inaccessible for me
 * (they had mode 0001), so I had to ask sysadmin to delete them from root account
 * Be careful :-)
 * <?php
 * fn_copy_dir('./testSRC','D:/srv/Apache2/htdocs/testDEST',0777,true);
 * ?>
 * 
 * 
 * 
 */
function fn_copy_dir($fromDir, $toDir, $chmod = 0757, $verbose = false)
{
	//* Check for some errors
	$errors = array();
	$messages = array();
	if (  !is_writable($toDir) )
		$errors[] = 'target ' . $toDir . ' is not writable';
	if (  !is_dir($toDir) )
		$errors[] = 'target ' . $toDir . ' is not a directory';
	if (  !is_dir($fromDir) )
		$errors[] = 'source ' . $fromDir . ' is not a directory';
	if (  !empty($errors) )
	{
		if ( $verbose )
			foreach ( $errors as $err )
				echo '<strong>Error</strong>: ' . $err . '<br />';
		return false;
	}
	//*/
	$exceptions = array('.','..');
	//* Processing
	$handle = opendir($fromDir);
	while (false !== ($item = readdir($handle)))
		if (  !in_array($item,$exceptions) )
		{
			//* cleanup for trailing slashes in directories destinations
			$from = str_replace('//','/',$fromDir . '/' . $item);
			$to = str_replace('//','/',$toDir . '/' . $item);
			//*/
			if ( is_file($from) )
			{
				if ( @ copy($from,$to) )
				{
					@chmod($to,$chmod);
					@touch($to,filemtime($from)); // to track last modified time
					$messages[] = 'File copied from ' . $from . ' to ' . $to;
				}
				else
					$errors[] = 'cannot copy file from ' . $from . ' to ' . $to;
			}
			if ( is_dir($from) )
			{
				if ( @ mkdir($to) )
				{
					@chmod($to,$chmod);
					$messages[] = 'Directory created: ' . $to;
				}
				else
					$errors[] = 'cannot create directory ' . $to;
				fn_copy_dir($from,$to,$chmod,$verbose);
			}
		}
	closedir($handle);
	//*/
	//* Output
	if ( $verbose )
	{
		foreach ( $errors as $err )
			echo '<strong>Error</strong>: ' . $err . '<br />';
		foreach ( $messages as $msg )
			echo $msg . '<br />';
	}
	//*/
	return true;
}
/**
 * ricava l' id di una sezione
 *
 * @param unknown_type $module
 * @return unknown
 */
function get_section_id($module = "")
{
	global $_FN;
	if ( $module == "" )
		$module = $_FN['vmod'];
	$g = glob("sections/$module/*.section");
	if ( isset($g[0]) )
		return preg_replace('/.section$/s','',basename($g[0]));
	else
	{
		$tmp = basename($module);
		$tmp = str_replace("none_","",$tmp);
		$tmp = preg_replace("/^[0-9][0-9]_/si","",$tmp);
		return basename($tmp);
	}
}
/**
 * cerca una sezione dato l' id
 * @param id
 * 
 */
function find_section($id)
{
	$dest = sectionlocation($id);
	if ( $dest == "" )
	{
		$dirs = list_subsections("sections",true,true);
		foreach ( $dirs as $dir )
		{
			if ( get_section_id("$dir") == $id )
			{
				return $dir;
			}
		}
	}
	return $dest;
}
/**
 * Funzione che maschera un indirizzo email
 *
 * Permette di codificare un indirizzo email laciato in chiaro,
 * di modo che non sia facilmente interpretabile dai bot.
 *
 * @author Andrea Biondo <andrearobot@hotmail.com>
 *
 * @param string $email_address	indirizzo email da codificare
 * @return string indirizzo email codificato
 */
function email_mask($email_address)
{
	$email_address = getparam($email_address,PAR_NULL,SAN_FLAT);
	$email_replace = array("@"=>"%20&#040;at&#041;%20","."=>"%20&#040;dot&#041;%20");
	foreach ( $email_replace as $chr=>$rep )
	{
		$email_address = str_replace($chr,$rep,$email_address);
	}
	return $email_address;
}
/**
 * Funzione verifica se un utente appartiene ad un gruppo
 *
 * @param string $user	utente
 * @return string $group gruppo
 */
function user_in_group($user, $group)
{
	$user = get_user($user);
	if ( isset($user['group']) )
	{
		$groups = explode(",",$user['group']);
		if ( is_array($groups) )
			return in_array($group,$groups);
	}
	return false;
}
/**
 * esegue il logout
 * 
 * 
 */
function fn_logout()
{
	global $urlcookie,$_FNUSER;
	if ( $urlcookie == "" )
	{
		$url = getparam("PHP_SELF",PAR_SERVER,SAN_FLAT);
		$path = pathinfo($url);
		$url = $path["dirname"];
		if ( $url == "" || $url == "\\" )
			$url = "/";
		$urlcookie = $url;
	}
	setcookie("secid","",NULL,"$urlcookie");
	setcookie("myforum","",NULL,"$urlcookie");
	setcookie("secid","",NULL,"/");
	setcookie("myforum","",NULL,"/");
	$_FNUSER = "";
	unset($_COOKIE['myforum']);
	unset($_COOKIE['secid']);
}
/**
 * esegue il login
 *
 * @param string $username
 * @return bool
 */
function fn_login($username, $password = "")
{
	global $_FN,$_FNUSER;
	if ( $password == "" )
	{
		$password = getpass($username);
	}
	else
	{
		$password=md5($password);
	}
	$us = get_user($username,false);
	if ( $us['active'] == 1 )
	{
		// imposto bene il path del cookie!
		$url = $_SERVER["PHP_SELF"];
		$path = pathinfo($url);
		$url = $path["dirname"];
		if ( $url == "" || $url == "\\" )
			$url = "/";
		$rememberlogin = getparam("rememberlogin",PAR_POST,SAN_FLAT);
		if ( $_FN['remember_login'] == 0 || $rememberlogin == "" )
		{
			setcookie("myforum",$username,0,"$url");
			setcookie("secid",md5($username . $password),0,"$url");
		}
		else
		{
			setcookie("myforum",$username,time() + 99999999,"$url");
			setcookie("secid",md5($username . $password),time() + 99999999,"$url");
		}
		FN_log("User $username login (function fn_login).");
		//forzo i cookie e tutto il resto
		$_FN['user'] = $_FNUSER = $_COOKIE['myforum'] = $username;
		$_COOKIE['secid'] = md5($username . $password);
		return true;
	}
	else
	{
		return false;
	}
}
/**
 * load files in the folder
 *
 * @param string $dir
 * @param string $extension
 * @return array
 */
function FN_LoadFilelist($dir, $extension = "php")
{
	if (  !file_exists($dir) ||  !is_dir($dir) )
		return array();
	$handle = opendir($dir);
	$ret = array();
	while (false !== ($file = readdir($handle)))
	{
		if ( $file != "." && $file != ".." && ( !preg_match('/^\./s',$file) && ($file != "CVS")) &&  !is_dir("$dir/$file") )
		{
			if (  !preg_match("/^none_/si",$file) && preg_match('/\.' . $extension . '$/s',$file) )
			{
				$ret[] = $dir . "/" . $file;
			}
		}
	}
	fn_natsort($ret);
	return $ret;
}
/**
 * load the contents of the folder
 *
 * @param string $dir
 * @return array
 */
function FN_LoadDirlist($dir)
{
	if (  !file_exists($dir) ||  !is_dir($dir) )
		return array();
	$handle = opendir($dir);
	$ret = array();
	while (false !== ($file = readdir($handle)))
	{
		if ( $file != "." && $file != ".." &&  !preg_match('/^\./s',$file) && is_dir("$dir/$file") )
		{
			if (  !preg_match("/^none_/si",$file) )
			{
				$ret[] = $dir . "/" . $file;
			}
		}
	}
	fn_natsort($ret);
	return $ret;
}
?>